@@ -1,6 +1,6 @@
 <?xml version="1.0" encoding="utf-8"?>
 <!DOCTYPE TS>
-<TS version="2.0">
+<TS version="2.1">
 <context>
     <name>About</name>
     <message>
@@ -15,10 +15,6 @@
         <source>Cppcheck - A tool for static C/C++ code analysis.</source>
         <translation>Cppcheck - 정적 C/C++ 코드 분석 도구.</translation>
     </message>
-    <message>
-        <source>Copyright © 2007-2012 Daniel Marjamäki and cppcheck team.</source>
-        <translation type="obsolete">Copyright © 2007-2012 Daniel Marjamäki and cppcheck team.</translation>
-    </message>
     <message>
         <source>This program is licensed under the terms
 of the GNU General Public License version 3</source>
@@ -29,13 +25,9 @@ of the GNU General Public License version 3</source>
         <source>Visit Cppcheck homepage at %1</source>
         <translation>Cppcheck 홈페이지(%1)를 방문해보세요</translation>
     </message>
-    <message utf8="true">
-        <source>Copyright © 2007-2013 Daniel Marjamäki and cppcheck team.</source>
-        <translation type="obsolete">Copyright © 2007-2012 Daniel Marjamäki and cppcheck team. {2007-2013 ?}</translation>
-    </message>
-    <message utf8="true">
-        <source>Copyright © 2007-2015 Daniel Marjamäki and cppcheck team.</source>
-        <translation type="unfinished">Copyright © 2007-2012 Daniel Marjamäki and cppcheck team. {2007-2013 ?} {2007-2015 ?}</translation>
+    <message>
+        <source>Copyright © 2007-2018 Cppcheck team.</source>
+        <translation type="unfinished"></translation>
     </message>
 </context>
 <context>
@@ -209,6 +201,30 @@ Kate로 파일을 열고, 해당 행으로 이동하는 예제:
         <source>Comments</source>
         <translation type="unfinished"></translation>
     </message>
+    <message>
+        <source>Save as</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Cppcheck</source>
+        <translation type="unfinished">Cppcheck</translation>
+    </message>
+    <message>
+        <source>Can not open file %1.</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Can not save file %1.</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Save the library as</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Failed to load %1. %2.</source>
+        <translation type="unfinished"></translation>
+    </message>
 </context>
 <context>
     <name>LibraryEditArgDialog</name>
@@ -270,10 +286,6 @@ Kate로 파일을 열고, 해당 행으로 이동하는 예제:
         <source>argvalue</source>
         <translation type="unfinished"></translation>
     </message>
-    <message>
-        <source>constant</source>
-        <translation type="unfinished"></translation>
-    </message>
     <message>
         <source>mul</source>
         <translation type="unfinished"></translation>
@@ -303,27 +315,27 @@ Kate로 파일을 열고, 해당 행으로 이동하는 예제:
     <name>LogView</name>
     <message>
         <source>Checking Log</source>
-        <translation>로그 확인</translation>
+        <translation type="vanished">로그 확인</translation>
     </message>
     <message>
         <source>Clear</source>
-        <translation>지우기</translation>
+        <translation type="vanished">지우기</translation>
     </message>
     <message>
         <source>Save Log</source>
-        <translation>로그 저장</translation>
+        <translation type="vanished">로그 저장</translation>
     </message>
     <message>
         <source>Text files (*.txt *.log);;All files (*.*)</source>
-        <translation>텍스트 파일 (*.txt *.log);;모든 파일 (*.*)</translation>
+        <translation type="vanished">텍스트 파일 (*.txt *.log);;모든 파일 (*.*)</translation>
     </message>
     <message>
         <source>Cppcheck</source>
-        <translation>Cppcheck</translation>
+        <translation type="vanished">Cppcheck</translation>
     </message>
     <message>
         <source>Could not open file for writing: &quot;%1&quot;</source>
-        <translation>기록할 파일 열기 실패: &quot;%1&quot;</translation>
+        <translation type="vanished">기록할 파일 열기 실패: &quot;%1&quot;</translation>
     </message>
 </context>
 <context>
@@ -350,7 +362,7 @@ Kate로 파일을 열고, 해당 행으로 이동하는 예제:
     </message>
     <message>
         <source>&amp;Check</source>
-        <translation>검사(&amp;C)</translation>
+        <translation type="obsolete">검사(&amp;C)</translation>
     </message>
     <message>
         <source>&amp;Edit</source>
@@ -386,7 +398,7 @@ Kate로 파일을 열고, 해당 행으로 이동하는 예제:
     </message>
     <message>
         <source>Check files</source>
-        <translation>파일 검사</translation>
+        <translation type="obsolete">파일 검사</translation>
     </message>
     <message>
         <source>Ctrl+F</source>
@@ -398,7 +410,7 @@ Kate로 파일을 열고, 해당 행으로 이동하는 예제:
     </message>
     <message>
         <source>Check directory</source>
-        <translation>디렉토리 검사</translation>
+        <translation type="obsolete">디렉토리 검사</translation>
     </message>
     <message>
         <source>Ctrl+D</source>
@@ -418,7 +430,7 @@ Kate로 파일을 열고, 해당 행으로 이동하는 예제:
     </message>
     <message>
         <source>Stop checking</source>
-        <translation>검사 중지</translation>
+        <translation type="obsolete">검사 중지</translation>
     </message>
     <message>
         <source>Esc</source>
@@ -446,7 +458,7 @@ Kate로 파일을 열고, 해당 행으로 이동하는 예제:
     </message>
     <message>
         <source>Style warnings</source>
-        <translation>스타일 경고</translation>
+        <translation type="vanished">스타일 경고</translation>
     </message>
     <message>
         <source>Show style warnings</source>
@@ -454,7 +466,7 @@ Kate로 파일을 열고, 해당 행으로 이동하는 예제:
     </message>
     <message>
         <source>Errors</source>
-        <translation>에러</translation>
+        <translation type="vanished">에러</translation>
     </message>
     <message>
         <source>Show errors</source>
@@ -542,7 +554,7 @@ Kate로 파일을 열고, 해당 행으로 이동하는 예제:
     </message>
     <message>
         <source>Warnings</source>
-        <translation>경고</translation>
+        <translation type="vanished">경고</translation>
     </message>
     <message>
         <source>Show warnings</source>
@@ -550,7 +562,7 @@ Kate로 파일을 열고, 해당 행으로 이동하는 예제:
     </message>
     <message>
         <source>Performance warnings</source>
-        <translation>성능 경고</translation>
+        <translation type="vanished">성능 경고</translation>
     </message>
     <message>
         <source>Show performance warnings</source>
@@ -570,7 +582,7 @@ Kate로 파일을 열고, 해당 행으로 이동하는 예제:
     </message>
     <message>
         <source>Portability</source>
-        <translation>이식성 경고</translation>
+        <translation type="vanished">이식성 경고</translation>
     </message>
     <message>
         <source>Show portability warnings</source>
@@ -606,19 +618,19 @@ Kate로 파일을 열고, 해당 행으로 이동하는 예제:
     </message>
     <message>
         <source>Platforms</source>
-        <translation>플랫폼</translation>
+        <translation type="vanished">플랫폼</translation>
     </message>
     <message>
         <source>C++11</source>
-        <translation>C++11</translation>
+        <translation type="vanished">C++11</translation>
     </message>
     <message>
         <source>C99</source>
-        <translation>C99</translation>
+        <translation type="vanished">C99</translation>
     </message>
     <message>
         <source>Posix</source>
-        <translation>Posix</translation>
+        <translation type="vanished">Posix</translation>
     </message>
     <message>
         <source>Quick Filter:</source>
@@ -634,7 +646,7 @@ Cppcheck 버전간 설정 방법 차이때문인 것으로 보입니다. 편집
     </message>
     <message>
         <source>No suitable files found to check!</source>
-        <translation>검사할 수 있는 파일이 없습니다!</translation>
+        <translation type="obsolete">검사할 수 있는 파일이 없습니다!</translation>
     </message>
     <message>
         <source>You must close the project file before selecting new files or directories!</source>
@@ -642,7 +654,7 @@ Cppcheck 버전간 설정 방법 차이때문인 것으로 보입니다. 편집
     </message>
     <message>
         <source>Select directory to check</source>
-        <translation>검사할 디렉토리 선택</translation>
+        <translation type="obsolete">검사할 디렉토리 선택</translation>
     </message>
     <message>
         <source>Found project file: %1
@@ -656,7 +668,7 @@ Do you want to load this project file instead?</source>
         <source>Found project files from the directory.
 
 Do you want to proceed checking without using any of these project files?</source>
-        <translation>디렉토리에 프로젝트 파일 존재.
+        <translation type="obsolete">디렉토리에 프로젝트 파일 존재.
 
 이 프로젝트 파일을 사용하지 않고 검사를 계속하시겠습니까?</translation>
     </message>
@@ -672,7 +684,7 @@ Do you want to proceed checking without using any of these project files?</sourc
         <source>Checking is running.
 
 Do you want to stop the checking and exit Cppcheck?</source>
-        <translation>검사 중.
+        <translation type="obsolete">검사 중.
 
 검사를 중지하고 Cppcheck을 종료하시겠습니까?</translation>
     </message>
@@ -686,7 +698,7 @@ Do you want to stop the checking and exit Cppcheck?</source>
     </message>
     <message>
         <source>XML files version 2 (*.xml);;XML files version 1 (*.xml);;Text files (*.txt);;CSV files (*.csv)</source>
-        <translation>XML 파일 버전 2 (*.xml);;XML 파일 버전 1 (*.xml);;텍스트 파일 (*.txt);;CSV 파일 (*.csv)</translation>
+        <translation type="obsolete">XML 파일 버전 2 (*.xml);;XML 파일 버전 1 (*.xml);;텍스트 파일 (*.txt);;CSV 파일 (*.csv)</translation>
     </message>
     <message>
         <source>Save the report file</source>
@@ -694,11 +706,11 @@ Do you want to stop the checking and exit Cppcheck?</source>
     </message>
     <message>
         <source>XML files version 1 (*.xml)</source>
-        <translation>XML 파일 버전 1 (*.xml)</translation>
+        <translation type="obsolete">XML 파일 버전 1 (*.xml)</translation>
     </message>
     <message>
         <source>XML files version 2 (*.xml)</source>
-        <translation>XML 파일 버전 2 (*.xml)</translation>
+        <translation type="obsolete">XML 파일 버전 2 (*.xml)</translation>
     </message>
     <message>
         <source>Text files (*.txt)</source>
@@ -766,13 +778,9 @@ Do you want to remove the file from the recently used projects -list?</source>
 Opening a new XML file will clear current results.Do you want to proceed?</source>
         <translation type="unfinished"></translation>
     </message>
-    <message>
-        <source>Show S&amp;cratchpad...</source>
-        <translation type="unfinished"></translation>
-    </message>
     <message>
         <source>Select files to check</source>
-        <translation type="unfinished">검사할 파일 선택</translation>
+        <translation type="obsolete">검사할 파일 선택</translation>
     </message>
     <message>
         <source>Cppcheck GUI - Command line parameters</source>
@@ -782,45 +790,22 @@ Opening a new XML file will clear current results.Do you want to proceed?</sourc
         <source>C++ standard</source>
         <translation type="unfinished"></translation>
     </message>
-    <message>
-        <source>C standard</source>
-        <translation type="unfinished"></translation>
-    </message>
     <message>
         <source>C11</source>
-        <translation type="unfinished">C11</translation>
+        <translation type="obsolete">C11</translation>
     </message>
     <message>
         <source>C89</source>
-        <translation type="unfinished">C89</translation>
+        <translation type="obsolete">C89</translation>
     </message>
     <message>
         <source>C++03</source>
-        <translation type="unfinished">C++03</translation>
+        <translation type="obsolete">C++03</translation>
     </message>
     <message>
         <source>Error</source>
         <translation type="unfinished"></translation>
     </message>
-    <message>
-        <source>Cppcheck GUI.
-
-Syntax:
-    cppcheck-gui [OPTIONS] [files or paths]
-
-Options:
-    -h, --help              Print this help
-    -p &lt;file&gt;               Open given project file and start checking it
-    -l &lt;file&gt;               Open given results xml file
-    -d &lt;directory&gt;          Specify the directory that was checked to generate the results xml specified with -l
-    -v, --version           Show program version
-    --data-dir=&lt;directory&gt;  Specify directory where GUI datafiles are located (translations, cfg)</source>
-        <translation type="unfinished"></translation>
-    </message>
-    <message>
-        <source>Failed to load %1. Your Cppcheck installation is broken. You can use --data-dir=&lt;directory&gt; at the command line to specify where this file is located.</source>
-        <translation type="unfinished"></translation>
-    </message>
     <message>
         <source>File not found</source>
         <translation type="unfinished"></translation>
@@ -875,36 +860,205 @@ Options:
         <source>Open a Print Preview Dialog for the Current Results</source>
         <translation type="unfinished"></translation>
     </message>
-    <message>
-        <source>Library Editor...</source>
-        <translation type="unfinished"></translation>
-    </message>
     <message>
         <source>Open library editor</source>
         <translation type="unfinished"></translation>
     </message>
     <message>
-        <source>Auto-detect language</source>
+        <source>Unknown element</source>
         <translation type="unfinished"></translation>
     </message>
     <message>
-        <source>Enforce C++</source>
+        <source>Unknown issue</source>
         <translation type="unfinished"></translation>
     </message>
     <message>
-        <source>Enforce C</source>
+        <source>C/C++ Source, Compile database, Visual Studio (%1 %2 *.sln *.vcxproj)</source>
         <translation type="unfinished"></translation>
     </message>
     <message>
-        <source>&amp;Recheck modified files</source>
+        <source>Select configuration</source>
         <translation type="unfinished"></translation>
     </message>
     <message>
-        <source>&amp;Recheck all files</source>
+        <source>Cppcheck GUI.
+
+Syntax:
+    cppcheck-gui [OPTIONS] [files or paths]
+
+Options:
+    -h, --help              Print this help
+    -p &lt;file&gt;               Open given project file and start checking it
+    -l &lt;file&gt;               Open given results xml file
+    -d &lt;directory&gt;          Specify the directory that was checked to generate the results xml specified with -l
+    -v, --version           Show program version
+    --data-dir=&lt;directory&gt;  This option is for installation scripts so they can configure the directory where
+                            datafiles are located (translations, cfg). The GUI is not started when this option
+                            is used.</source>
         <translation type="unfinished"></translation>
     </message>
     <message>
-        <source>C/C++ Source (%1)</source>
+        <source>Failed to load %1. Your Cppcheck installation is broken. You can use --data-dir=&lt;directory&gt; at the command line to specify where this file is located. Please note that --data-dir is supposed to be used by installation scripts and therefore the GUI does not start when it is used, all that happens is that the setting is configured.</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Build dir &apos;%1&apos; does not exist, create it?</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Analyze files</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Analyze directory</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>&amp;Reanalyze modified files</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Stop analysis</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>XML files (*.xml);;Text files (*.txt);;CSV files (*.csv)</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>No suitable files found to analyze!</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Select files to analyze</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Select directory to analyze</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Select the configuration that will be analyzed</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Found project files from the directory.
+
+Do you want to proceed analysis without using any of these project files?</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Analyzer is running.
+
+Do you want to stop the analysis and exit Cppcheck?</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>A&amp;nalyze</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>&amp;C standard</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Reanal&amp;yze all files</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Style war&amp;nings</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>E&amp;rrors</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Sh&amp;ow Scratchpad...</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>&amp;Warnings</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Per&amp;formance warnings</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>&amp;Information</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>&amp;Portability</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Show Cppcheck results</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Clang</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Show Clang results</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>P&amp;latforms</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>C++&amp;11</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>C&amp;99</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>&amp;Posix</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>C&amp;11</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>&amp;C89</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>&amp;C++03</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>&amp;Library Editor...</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>&amp;Auto-detect language</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>&amp;Enforce C++</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>E&amp;nforce C</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>C++14</source>
+        <translation type="unfinished">C++14</translation>
+    </message>
+    <message>
+        <source>Failed to import &apos;%1&apos;, analysis is stopped</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Project files (*.cppcheck)</source>
         <translation type="unfinished"></translation>
     </message>
 </context>
@@ -932,22 +1086,26 @@ Options:
     </message>
     <message>
         <source>Built-in</source>
-        <translation>내장 방식</translation>
+        <translation type="obsolete">내장 방식</translation>
+    </message>
+    <message>
+        <source>Native</source>
+        <translation type="unfinished"></translation>
     </message>
 </context>
 <context>
     <name>Project</name>
     <message>
         <source>Cppcheck</source>
-        <translation>Cppcheck</translation>
+        <translation type="vanished">Cppcheck</translation>
     </message>
     <message>
         <source>Could not read the project file.</source>
-        <translation>프로젝트 파일을 읽을 수 없습니다.</translation>
+        <translation type="vanished">프로젝트 파일을 읽을 수 없습니다.</translation>
     </message>
     <message>
         <source>Could not write the project file.</source>
-        <translation>프로젝트 파일에 쓸 수 없습니다.</translation>
+        <translation type="vanished">프로젝트 파일에 쓸 수 없습니다.</translation>
     </message>
 </context>
 <context>
@@ -958,7 +1116,7 @@ Options:
     </message>
     <message>
         <source>Project</source>
-        <translation>프로젝트</translation>
+        <translation type="vanished">프로젝트</translation>
     </message>
     <message>
         <source>Defines:</source>
@@ -966,7 +1124,7 @@ Options:
     </message>
     <message>
         <source>Root:</source>
-        <translation>Root:</translation>
+        <translation type="obsolete">Root:</translation>
     </message>
     <message>
         <source>Paths:</source>
@@ -986,11 +1144,11 @@ Options:
     </message>
     <message>
         <source>Includes</source>
-        <translation>Includes</translation>
+        <translation type="obsolete">Includes</translation>
     </message>
     <message>
         <source>Include directories:</source>
-        <translation>Include 디렉토리:</translation>
+        <translation type="obsolete">Include 디렉토리:</translation>
     </message>
     <message>
         <source>Up</source>
@@ -1002,20 +1160,12 @@ Options:
     </message>
     <message>
         <source>Exclude</source>
-        <translation>Exclude</translation>
-    </message>
-    <message>
-        <source>Libraries:</source>
-        <translation type="unfinished"></translation>
+        <translation type="vanished">Exclude</translation>
     </message>
     <message>
         <source>Suppressions</source>
         <translation type="unfinished"></translation>
     </message>
-    <message>
-        <source>Suppression list:</source>
-        <translation type="unfinished"></translation>
-    </message>
     <message>
         <source>Add</source>
         <translation type="unfinished"></translation>
@@ -1025,7 +1175,115 @@ Options:
         <translation type="unfinished"></translation>
     </message>
     <message>
-        <source>Defines must be separated by a semicolon &apos;;&apos;</source>
+        <source>...</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Include Paths:</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Paths and Defines</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Import Project (Visual studio / compile database)</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>&lt;html&gt;&lt;head/&gt;&lt;body&gt;&lt;p&gt;You have a choice:&lt;/p&gt;&lt;p&gt; * Analyze all Debug and Release configurations&lt;/p&gt;&lt;p&gt; * Only analyze the first matching Debug configuration&lt;/p&gt;&lt;p&gt;&lt;br/&gt;&lt;/p&gt;&lt;/body&gt;&lt;/html&gt;</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Analyze all Visual Studio configurations</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Root path:</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Warning tags (separated by semicolon)</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Cppcheck build dir (whole program analysis, incremental analysis, statistics, etc)</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Libraries</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Addons</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Y2038</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Thread safety</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Coding standards</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Cert</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Clang analyzer</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Clang-tidy</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Browse...</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Defines must be separated by a semicolon. Example: DEF1;DEF2=5;DEF3=int</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Checking</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Platform</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Warning options</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Exclude paths</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Addons and tools</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>MISRA C 2012</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Misra rule texts</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>&lt;html&gt;&lt;head/&gt;&lt;body&gt;&lt;p&gt;Copy/paste the text from Appendix A &amp;quot;Summary of guidelines&amp;quot; from the MISRA C 2012 pdf to a text file.&lt;/p&gt;&lt;/body&gt;&lt;/html&gt;</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>External tools</source>
         <translation type="unfinished"></translation>
     </message>
 </context>
@@ -1055,6 +1313,34 @@ Options:
         <source>Select error id suppress:</source>
         <translation type="unfinished"></translation>
     </message>
+    <message>
+        <source>Select Cppcheck build dir</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Import Project</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Visual Studio (*.sln *.vcxproj);;Compile database (compile_commands.json)</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Clang-tidy (not found)</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>(no rule texts file)</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Select MISRA rule texts file</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Misra rule texts file (%1)</source>
+        <translation type="unfinished"></translation>
+    </message>
 </context>
 <context>
     <name>QDialogButtonBox</name>
@@ -1089,6 +1375,14 @@ Options:
         <source>Failed to load translation for language %1 from file %2</source>
         <translation>파일(%2)로부터 언어(%1) 불러오기 실패</translation>
     </message>
+    <message>
+        <source>line %1: Unhandled element %2</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source> (Not found)</source>
+        <translation type="unfinished"></translation>
+    </message>
 </context>
 <context>
     <name>QPlatformTheme</name>
@@ -1215,7 +1509,7 @@ Please check the application path and parameters are correct.</source>
         <source>Could not find file:
 %1
 Please select the directory where file is located.</source>
-        <translation>파일 찾기 실패:
+        <translation type="vanished">파일 찾기 실패:
 %1
 파일이 위치한 디렉토리를 선택하세요.</translation>
     </message>
@@ -1243,6 +1537,50 @@ Please select the directory where file is located.</source>
         <source>Inconclusive</source>
         <translation type="unfinished"></translation>
     </message>
+    <message>
+        <source>Recheck</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>note</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Suppress selected id(s)</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Tag</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>No tag</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Since date</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Copy line nr</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Could not find file:</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Please select the folder &apos;%1&apos;</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Select Directory &apos;%1&apos;</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Please select the directory where file is located.</source>
+        <translation type="unfinished"></translation>
+    </message>
 </context>
 <context>
     <name>ResultsView</name>
@@ -1304,6 +1642,30 @@ To toggle what kind of errors are shown, open view menu.</source>
         <source>First included by</source>
         <translation type="unfinished"></translation>
     </message>
+    <message>
+        <source>XML format version 1 is no longer supported.</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Analysis Log</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Warning Details</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Clear Log</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Copy this Log entry</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Copy complete Log</source>
+        <translation type="unfinished"></translation>
+    </message>
 </context>
 <context>
     <name>ScratchPad</name>
@@ -1319,6 +1681,14 @@ To toggle what kind of errors are shown, open view menu.</source>
         <source>Check</source>
         <translation type="unfinished">검사</translation>
     </message>
+    <message>
+        <source>Copy or write some C/C++ code here:</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Optionally enter a filename (mainly for automatic language detection) and click on &quot;Check&quot;:</source>
+        <translation type="unfinished"></translation>
+    </message>
 </context>
 <context>
     <name>Settings</name>
@@ -1356,11 +1726,11 @@ To toggle what kind of errors are shown, open view menu.</source>
     </message>
     <message>
         <source>Paths</source>
-        <translation>경로</translation>
+        <translation type="vanished">경로</translation>
     </message>
     <message>
         <source>Include paths:</source>
-        <translation>Include 경로:</translation>
+        <translation type="vanished">Include 경로:</translation>
     </message>
     <message>
         <source>Add...</source>
@@ -1368,7 +1738,7 @@ To toggle what kind of errors are shown, open view menu.</source>
     </message>
     <message>
         <source>Edit</source>
-        <translation>편집</translation>
+        <translation type="vanished">편집</translation>
     </message>
     <message>
         <source>Remove</source>
@@ -1426,6 +1796,50 @@ To toggle what kind of errors are shown, open view menu.</source>
         <source>Show internal warnings in log</source>
         <translation type="unfinished"></translation>
     </message>
+    <message>
+        <source>Show statistics on check completion</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Addons</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Python binary (leave this empty to use python in the PATH)</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>...</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Clang</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Clang path (leave empty to use system PATH)</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Visual Studio headers</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>&lt;html&gt;&lt;head/&gt;&lt;body&gt;&lt;p&gt;Paths to Visual Studio headers, separated by semicolon &apos;;&apos;.&lt;/p&gt;&lt;p&gt;You can open a Visual Studio command prompt, write &amp;quot;SET INCLUDE&amp;quot;. Then copy/paste the paths.&lt;/p&gt;&lt;/body&gt;&lt;/html&gt;</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Misra addon</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Misra rule texts file</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>&lt;html&gt;&lt;head/&gt;&lt;body&gt;&lt;p&gt;Copy/paste the text from Appendix A &amp;quot;Summary of guidelines&amp;quot; from the MISRA C 2012 pdf to a text file.&lt;/p&gt;&lt;/body&gt;&lt;/html&gt;</source>
+        <translation type="unfinished"></translation>
+    </message>
 </context>
 <context>
     <name>SettingsDialog</name>
@@ -1447,7 +1861,23 @@ To toggle what kind of errors are shown, open view menu.</source>
     </message>
     <message>
         <source>Select include directory</source>
-        <translation>Include 디렉토리 선택</translation>
+        <translation type="vanished">Include 디렉토리 선택</translation>
+    </message>
+    <message>
+        <source> [Default]</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Select python binary</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Select clang path</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Select MISRA File</source>
+        <translation type="unfinished"></translation>
     </message>
 </context>
 <context>
@@ -1612,6 +2042,30 @@ To toggle what kind of errors are shown, open view menu.</source>
         <source>Information messages</source>
         <translation>정보 메시지</translation>
     </message>
+    <message>
+        <source>Pdf Export</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>Export PDF</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>History</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>File:</source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>File: </source>
+        <translation type="unfinished"></translation>
+    </message>
+    <message>
+        <source>No cppcheck build dir</source>
+        <translation type="unfinished"></translation>
+    </message>
 </context>
 <context>
     <name>ThreadResult</name>
