@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2015 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2017 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,10 +22,18 @@
 #define checkotherH
 //---------------------------------------------------------------------------
 
-#include "config.h"
 #include "check.h"
+#include "config.h"
+#include "valueflow.h"
 
-class Function;
+#include <cstddef>
+#include <string>
+#include <vector>
+
+class ErrorLogger;
+class Settings;
+class Token;
+class Tokenizer;
 class Variable;
 
 /// @addtogroup Checks
@@ -66,14 +74,13 @@ public:
         checkOther.checkVarFuncNullUB();
         checkOther.checkNanInArithmeticExpression();
         checkOther.checkCommaSeparatedReturn();
-        checkOther.checkIgnoredReturnValue();
         checkOther.checkRedundantPointerOp();
         checkOther.checkZeroDivision();
+        checkOther.checkNegativeBitwiseShift();
         checkOther.checkInterlockedDecrement();
         checkOther.checkUnusedLabel();
-
-        // --check-library : functions with nonmatching configuration
-        checkOther.checkLibraryMatchFunctions();
+        checkOther.checkEvaluationOrder();
+        checkOther.checkFuncArgNamesDifferent();
     }
 
     /** @brief Run checks against the simplified token list */
@@ -83,24 +90,18 @@ public:
         // Checks
         checkOther.clarifyCalculation();
         checkOther.clarifyStatement();
-        checkOther.checkConstantFunctionParameter();
+        checkOther.checkPassByReference();
         checkOther.checkIncompleteStatement();
         checkOther.checkCastIntToCharAndBack();
 
-        checkOther.invalidFunctionUsage();
-        checkOther.checkMathFunctions();
-
         checkOther.checkMisusedScopedObject();
-        checkOther.checkMemsetZeroBytes();
-        checkOther.checkMemsetInvalid2ndParam();
-        checkOther.checkSwitchCaseFallThrough();
         checkOther.checkPipeParameterSize();
 
         checkOther.checkInvalidFree();
         checkOther.checkRedundantCopy();
-        checkOther.checkNegativeBitwiseShift();
         checkOther.checkSuspiciousEqualityComparison();
         checkOther.checkComparisonFunctionIsAlwaysTrueOrFalse();
+        checkOther.checkAccessOfMovedVariable();
     }
 
     /** @brief Clarify calculation for ".. a * b ? .." */
@@ -115,16 +116,6 @@ public:
     /** @brief Check for pointer casts to a type with an incompatible binary data representation */
     void invalidPointerCast();
 
-    /**
-     * @brief Invalid function usage (invalid input value / overlapping data)
-     *
-     * %Check that given function parameters are valid according to the standard
-     * - wrong radix given for strtol/strtoul
-     * - overlapping data when using sprintf/snprintf
-     * - wrong input value according to library
-     */
-    void invalidFunctionUsage();
-
     /** @brief %Check scope of variables */
     void checkVariableScope();
     static bool checkInnerScope(const Token *tok, const Variable* var, bool& used);
@@ -132,8 +123,8 @@ public:
     /** @brief %Check for comma separated statements in return */
     void checkCommaSeparatedReturn();
 
-    /** @brief %Check for constant function parameter */
-    void checkConstantFunctionParameter();
+    /** @brief %Check for function parameters that should be passed by reference */
+    void checkPassByReference();
 
     /** @brief Using char variable as array index / as operand in bit operation */
     void checkCharVariable();
@@ -147,9 +138,6 @@ public:
     /** @brief Check for NaN (not-a-number) in an arithmetic expression */
     void checkNanInArithmeticExpression();
 
-    /** @brief %Check for parameters given to math function that do not make sense*/
-    void checkMathFunctions();
-
     /** @brief copying to memory or assigning to a variable twice */
     void checkRedundantAssignment();
 
@@ -162,18 +150,9 @@ public:
     /** @brief %Check for code like 'case A||B:'*/
     void checkSuspiciousEqualityComparison();
 
-    /** @brief %Check for switch case fall through without comment */
-    void checkSwitchCaseFallThrough();
-
     /** @brief %Check for objects that are destroyed immediately */
     void checkMisusedScopedObject();
 
-    /** @brief %Check for filling zero bytes with memset() */
-    void checkMemsetZeroBytes();
-
-    /** @brief %Check for invalid 2nd parameter of memset() */
-    void checkMemsetInvalid2ndParam();
-
     /** @brief %Check for suspicious code where if and else branch are the same (e.g "if (a) b = true; else b = true;") */
     void checkDuplicateBranch();
 
@@ -214,55 +193,50 @@ public:
     /** @brief %Check for using of comparison functions evaluating always to true or false. */
     void checkComparisonFunctionIsAlwaysTrueOrFalse();
 
-    /** @brief %Check for ignored return values. */
-    void checkIgnoredReturnValue();
-
     /** @brief %Check for redundant pointer operations */
     void checkRedundantPointerOp();
 
-    /** @brief --check-library: warn for unconfigured function calls */
-    void checkLibraryMatchFunctions();
-
     /** @brief %Check for race condition with non-interlocked access after InterlockedDecrement() */
     void checkInterlockedDecrement();
 
     /** @brief %Check for unused labels */
     void checkUnusedLabel();
 
+    /** @brief %Check for expression that depends on order of evaluation of side effects */
+    void checkEvaluationOrder();
+
+    /** @brief %Check for access of moved or forwarded variable */
+    void checkAccessOfMovedVariable();
+
+    /** @brief %Check if function declaration and definition argument names different */
+    void checkFuncArgNamesDifferent();
+
 private:
     // Error messages..
-    void checkComparisonFunctionIsAlwaysTrueOrFalseError(const Token* tok, const std::string &strFunctionName, const std::string &varName, const bool result);
+    void checkComparisonFunctionIsAlwaysTrueOrFalseError(const Token* tok, const std::string &functionName, const std::string &varName, const bool result);
     void checkCastIntToCharAndBackError(const Token *tok, const std::string &strFunctionName);
     void checkPipeParameterSizeError(const Token *tok, const std::string &strVarName, const std::string &strDim);
     void clarifyCalculationError(const Token *tok, const std::string &op);
     void clarifyStatementError(const Token* tok);
     void cstyleCastError(const Token *tok);
     void invalidPointerCastError(const Token* tok, const std::string& from, const std::string& to, bool inconclusive);
-    void invalidFunctionArgError(const Token *tok, const std::string &functionName, int argnr, const std::string &validstr);
-    void invalidFunctionArgBoolError(const Token *tok, const std::string &functionName, int argnr);
-    void passedByValueError(const Token *tok, const std::string &parname);
+    void passedByValueError(const Token *tok, const std::string &parname, bool inconclusive);
     void constStatementError(const Token *tok, const std::string &type);
-    void charArrayIndexError(const Token *tok);
+    void signedCharArrayIndexError(const Token *tok);
+    void unknownSignCharArrayIndexError(const Token *tok);
     void charBitOpError(const Token *tok);
     void variableScopeError(const Token *tok, const std::string &varname);
-    void zerodivError(const Token *tok, bool inconclusive);
-    void zerodivcondError(const Token *tokcond, const Token *tokdiv, bool inconclusive);
+    void zerodivError(const Token *tok, const ValueFlow::Value *value);
     void nanInArithmeticExpressionError(const Token *tok);
-    void mathfunctionCallWarning(const Token *tok, const unsigned int numParam = 1);
-    void mathfunctionCallWarning(const Token *tok, const std::string& oldexp, const std::string& newexp);
     void redundantAssignmentError(const Token *tok1, const Token* tok2, const std::string& var, bool inconclusive);
     void redundantAssignmentInSwitchError(const Token *tok1, const Token *tok2, const std::string &var);
     void redundantCopyError(const Token *tok1, const Token* tok2, const std::string& var);
     void redundantCopyInSwitchError(const Token *tok1, const Token* tok2, const std::string &var);
     void redundantBitwiseOperationInSwitchError(const Token *tok, const std::string &varname);
-    void switchCaseFallThrough(const Token *tok);
     void suspiciousCaseInSwitchError(const Token* tok, const std::string& operatorString);
     void suspiciousEqualityComparisonError(const Token* tok);
     void selfAssignmentError(const Token *tok, const std::string &varname);
     void misusedScopeObjectError(const Token *tok, const std::string &varname);
-    void memsetZeroBytesError(const Token *tok, const std::string &varname);
-    void memsetFloatError(const Token *tok, const std::string &var_value);
-    void memsetValueOutOfRangeError(const Token *tok, const std::string &value);
     void duplicateBranchError(const Token *tok1, const Token *tok2);
     void duplicateExpressionError(const Token *tok1, const Token *tok2, const std::string &op);
     void duplicateExpressionTernaryError(const Token *tok);
@@ -273,74 +247,77 @@ private:
     void unsignedPositiveError(const Token *tok, const std::string &varname, bool inconclusive);
     void pointerPositiveError(const Token *tok, bool inconclusive);
     void SuspiciousSemicolonError(const Token *tok);
-    void negativeBitwiseShiftError(const Token *tok);
+    void negativeBitwiseShiftError(const Token *tok, int op);
     void redundantCopyError(const Token *tok, const std::string &varname);
     void incompleteArrayFillError(const Token* tok, const std::string& buffer, const std::string& function, bool boolean);
     void varFuncNullUBError(const Token *tok);
     void commaSeparatedReturnError(const Token *tok);
-    void ignoredReturnValueError(const Token* tok, const std::string& function);
     void redundantPointerOpError(const Token* tok, const std::string& varname, bool inconclusive);
     void raceAfterInterlockedDecrementError(const Token* tok);
-    void unusedLabelError(const Token* tok);
+    void unusedLabelError(const Token* tok, bool inSwitch);
+    void unknownEvaluationOrder(const Token* tok);
+    static bool isMovedParameterAllowedForInconclusiveFunction(const Token * tok);
+    void accessMovedError(const Token *tok, const std::string &varname, const ValueFlow::Value *value, bool inconclusive);
+    void funcArgNamesDifferent(const std::string & functionName, size_t index, const Token* declaration, const Token* definition);
+    void funcArgOrderDifferent(const std::string & functionName, const Token * declaration, const Token * definition, const std::vector<const Token*> & declarations, const std::vector<const Token*> & definitions);
 
     void getErrorMessages(ErrorLogger *errorLogger, const Settings *settings) const {
-        CheckOther c(0, settings, errorLogger);
+        CheckOther c(nullptr, settings, errorLogger);
 
         // error
-        c.invalidFunctionArgError(0, "func_name", 1, "1-4");
-        c.invalidFunctionArgBoolError(0, "func_name", 1);
-        c.zerodivError(0, false);
-        c.zerodivcondError(0,0,false);
-        c.misusedScopeObjectError(NULL, "varname");
-        c.invalidPointerCastError(0, "float", "double", false);
-        c.negativeBitwiseShiftError(0);
-        c.checkPipeParameterSizeError(0, "varname", "dimension");
-        c.raceAfterInterlockedDecrementError(0);
+        c.zerodivError(nullptr, nullptr);
+        c.misusedScopeObjectError(nullptr, "varname");
+        c.invalidPointerCastError(nullptr,  "float", "double", false);
+        c.negativeBitwiseShiftError(nullptr, 1);
+        c.negativeBitwiseShiftError(nullptr, 2);
+        c.checkPipeParameterSizeError(nullptr,  "varname", "dimension");
+        c.raceAfterInterlockedDecrementError(nullptr);
 
         //performance
-        c.redundantCopyError(0, "varname");
-        c.redundantCopyError(0, 0, "var");
-        c.redundantAssignmentError(0, 0, "var", false);
+        c.redundantCopyError(nullptr,  "varname");
+        c.redundantCopyError(nullptr, nullptr, "var");
+        c.redundantAssignmentError(nullptr, nullptr, "var", false);
 
         // style/warning
-        c.checkComparisonFunctionIsAlwaysTrueOrFalseError(0,"isless","varName",false);
-        c.checkCastIntToCharAndBackError(0,"func_name");
-        c.cstyleCastError(0);
-        c.passedByValueError(0, "parametername");
-        c.constStatementError(0, "type");
-        c.charArrayIndexError(0);
-        c.charBitOpError(0);
-        c.variableScopeError(0, "varname");
-        c.redundantAssignmentInSwitchError(0, 0, "var");
-        c.redundantCopyInSwitchError(0, 0, "var");
-        c.switchCaseFallThrough(0);
-        c.suspiciousCaseInSwitchError(0, "||");
-        c.suspiciousEqualityComparisonError(0);
-        c.selfAssignmentError(0, "varname");
-        c.mathfunctionCallWarning(0);
-        c.mathfunctionCallWarning(0, "1 - erf(x)", "erfc(x)");
-        c.memsetZeroBytesError(0, "varname");
-        c.memsetFloatError(0, "varname");
-        c.memsetValueOutOfRangeError(0, "varname");
-        c.clarifyCalculationError(0, "+");
-        c.clarifyStatementError(0);
-        c.duplicateBranchError(0, 0);
-        c.duplicateExpressionError(0, 0, "&&");
-        c.duplicateExpressionTernaryError(0);
-        c.duplicateBreakError(0, false);
-        c.unreachableCodeError(0, false);
-        c.unsignedLessThanZeroError(0, "varname", false);
-        c.unsignedPositiveError(0, "varname", false);
-        c.pointerLessThanZeroError(0, false);
-        c.pointerPositiveError(0, false);
-        c.SuspiciousSemicolonError(0);
-        c.incompleteArrayFillError(0, "buffer", "memset", false);
-        c.varFuncNullUBError(0);
-        c.nanInArithmeticExpressionError(0);
-        c.commaSeparatedReturnError(0);
-        c.ignoredReturnValueError(0, "malloc");
-        c.redundantPointerOpError(0, "varname", false);
-        c.unusedLabelError(0);
+        c.checkComparisonFunctionIsAlwaysTrueOrFalseError(nullptr, "isless","varName",false);
+        c.checkCastIntToCharAndBackError(nullptr, "func_name");
+        c.cstyleCastError(nullptr);
+        c.passedByValueError(nullptr,  "parametername", false);
+        c.constStatementError(nullptr,  "type");
+        c.signedCharArrayIndexError(nullptr);
+        c.unknownSignCharArrayIndexError(nullptr);
+        c.charBitOpError(nullptr);
+        c.variableScopeError(nullptr,  "varname");
+        c.redundantAssignmentInSwitchError(nullptr, nullptr, "var");
+        c.redundantCopyInSwitchError(nullptr, nullptr, "var");
+        c.suspiciousCaseInSwitchError(nullptr,  "||");
+        c.suspiciousEqualityComparisonError(nullptr);
+        c.selfAssignmentError(nullptr,  "varname");
+        c.clarifyCalculationError(nullptr,  "+");
+        c.clarifyStatementError(nullptr);
+        c.duplicateBranchError(nullptr, nullptr);
+        c.duplicateExpressionError(nullptr, nullptr, "&&");
+        c.duplicateExpressionTernaryError(nullptr);
+        c.duplicateBreakError(nullptr,  false);
+        c.unreachableCodeError(nullptr,  false);
+        c.unsignedLessThanZeroError(nullptr,  "varname", false);
+        c.unsignedPositiveError(nullptr,  "varname", false);
+        c.pointerLessThanZeroError(nullptr,  false);
+        c.pointerPositiveError(nullptr,  false);
+        c.SuspiciousSemicolonError(nullptr);
+        c.incompleteArrayFillError(nullptr,  "buffer", "memset", false);
+        c.varFuncNullUBError(nullptr);
+        c.nanInArithmeticExpressionError(nullptr);
+        c.commaSeparatedReturnError(nullptr);
+        c.redundantPointerOpError(nullptr,  "varname", false);
+        c.unusedLabelError(nullptr,  true);
+        c.unusedLabelError(nullptr,  false);
+        c.unknownEvaluationOrder(nullptr);
+        c.accessMovedError(nullptr, "v", nullptr, false);
+        c.funcArgNamesDifferent("function", 1, nullptr, nullptr);
+
+        std::vector<const Token *> nullvec;
+        c.funcArgOrderDifferent("function", nullptr, nullptr, nullvec, nullvec);
     }
 
     static std::string myName() {
@@ -358,26 +335,24 @@ private:
                "- bitwise operation with negative right operand\n"
                "- provide wrong dimensioned array to pipe() system command (--std=posix)\n"
                "- cast the return values of getc(),fgetc() and getchar() to character and compare it to EOF\n"
-               "- invalid input values for functions\n"
                "- race condition with non-interlocked access after InterlockedDecrement() call\n"
+               "- expression 'x = x++;' depends on order of evaluation of side effects\n"
 
                // warning
                "- either division by zero or useless condition\n"
-               "- memset() with a value out of range as the 2nd parameter\n"
-               "- return value of certain functions not used\n"
+               "- access of moved or forwarded variable.\n"
 
                // performance
                "- redundant data copying for const variable\n"
                "- subsequent assignment or copying to a variable or buffer\n"
+               "- passing parameter by value\n"
 
                // portability
-               "- memset() with a float as the 2nd parameter\n"
                "- Passing NULL pointer to function with variable number of arguments leads to UB.\n"
 
                // style
                "- C-style pointer cast in C++ code\n"
                "- casting between incompatible pointer types\n"
-               "- passing parameter by value\n"
                "- [Incomplete statement](IncompleteStatement)\n"
                "- [check how signed char variables are used](CharVar)\n"
                "- variable scope can be limited\n"
@@ -399,7 +374,9 @@ private:
                "- prefer erfc, expm1 or log1p to avoid loss of precision.\n"
                "- identical code in both branches of if/else or ternary operator.\n"
                "- redundant pointer operation on pointer like &*some_ptr.\n"
-               "- find unused 'goto' labels.\n";
+               "- find unused 'goto' labels.\n"
+               "- function declaration and definition argument names different.\n"
+               "- function declaration and definition argument order different.\n";
     }
 };
 /// @}
