@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2015 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2018 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,16 +17,31 @@
  */
 
 #include "library.h"
-#include "path.h"
-#include "tinyxml2.h"
-#include "tokenlist.h"
-#include "mathlib.h"
-#include "token.h"
-#include "symboldatabase.h"
-#include "astutils.h"
 
-#include <string>
-#include <algorithm>
+#include "astutils.h"
+#include "mathlib.h"
+#include "path.h"
+#include "symboldatabase.h"
+#include "tinyxml2.h"
+#include "token.h"
+#include "tokenlist.h"
+#include "utils.h"
+
+#include <cctype>
+#include <cstdlib>
+#include <cstring>
+#include <list>
+
+static std::vector<std::string> getnames(const char *names)
+{
+    std::vector<std::string> ret;
+    while (const char *p = std::strchr(names,',')) {
+        ret.push_back(std::string(names, p-names));
+        names = p + 1;
+    }
+    ret.push_back(names);
+    return ret;
+}
 
 Library::Library() : allocid(0)
 {
@@ -57,32 +72,36 @@ Library::Error Library::load(const char exename[], const char path[])
     if (error == tinyxml2::XML_ERROR_FILE_NOT_FOUND) {
         // failed to open file.. is there no extension?
         std::string fullfilename(path);
-        if (Path::getFilenameExtension(fullfilename) == "") {
+        if (Path::getFilenameExtension(fullfilename).empty()) {
             fullfilename += ".cfg";
             error = doc.LoadFile(fullfilename.c_str());
             if (error != tinyxml2::XML_ERROR_FILE_NOT_FOUND)
-                absolute_path = Path::getAbsoluteFilePath(fullfilename.c_str());
+                absolute_path = Path::getAbsoluteFilePath(fullfilename);
         }
 
-        if (error == tinyxml2::XML_ERROR_FILE_NOT_FOUND) {
-            // Try to locate the library configuration in the installation folder..
+        std::list<std::string> cfgfolders;
 #ifdef CFGDIR
-            const std::string cfgfolder(CFGDIR);
-#else
-            if (!exename)
-                return Error(FILE_NOT_FOUND);
-            const std::string cfgfolder(Path::fromNativeSeparators(Path::getPathFromFilename(exename)) + "cfg");
+        cfgfolders.push_back(CFGDIR);
 #endif
-            const char *sep = (!cfgfolder.empty() && cfgfolder[cfgfolder.size()-1U]=='/' ? "" : "/");
+        if (exename) {
+            const std::string exepath(Path::fromNativeSeparators(Path::getPathFromFilename(exename)));
+            cfgfolders.push_back(exepath + "cfg");
+            cfgfolders.push_back(exepath);
+        }
+
+        while (error == tinyxml2::XML_ERROR_FILE_NOT_FOUND && !cfgfolders.empty()) {
+            const std::string cfgfolder(cfgfolders.front());
+            cfgfolders.pop_front();
+            const char *sep = (!cfgfolder.empty() && endsWith(cfgfolder,'/') ? "" : "/");
             const std::string filename(cfgfolder + sep + fullfilename);
             error = doc.LoadFile(filename.c_str());
             if (error != tinyxml2::XML_ERROR_FILE_NOT_FOUND)
-                absolute_path = Path::getAbsoluteFilePath(filename.c_str());
+                absolute_path = Path::getAbsoluteFilePath(filename);
         }
     } else
         absolute_path = Path::getAbsoluteFilePath(path);
 
-    if (error == tinyxml2::XML_NO_ERROR) {
+    if (error == tinyxml2::XML_SUCCESS) {
         if (_files.find(absolute_path) == _files.end()) {
             Error err = load(doc);
             if (err.errorcode == OK)
@@ -99,7 +118,7 @@ Library::Error Library::load(const char exename[], const char path[])
 bool Library::loadxmldata(const char xmldata[], std::size_t len)
 {
     tinyxml2::XMLDocument doc;
-    return (tinyxml2::XML_NO_ERROR == doc.Parse(xmldata, len)) && (load(doc).errorcode == OK);
+    return (tinyxml2::XML_SUCCESS == doc.Parse(xmldata, len)) && (load(doc).errorcode == OK);
 }
 
 Library::Error Library::load(const tinyxml2::XMLDocument &doc)
@@ -129,9 +148,9 @@ Library::Error Library::load(const tinyxml2::XMLDocument &doc)
             int allocationId = 0;
             for (const tinyxml2::XMLElement *memorynode = node->FirstChildElement(); memorynode; memorynode = memorynode->NextSiblingElement()) {
                 if (strcmp(memorynode->Name(),"dealloc")==0) {
-                    const std::map<std::string,int>::const_iterator it = _dealloc.find(memorynode->GetText());
+                    const std::map<std::string, AllocFunc>::const_iterator it = _dealloc.find(memorynode->GetText());
                     if (it != _dealloc.end()) {
-                        allocationId = it->second;
+                        allocationId = it->second.groupId;
                         break;
                     }
                 }
@@ -148,15 +167,29 @@ Library::Error Library::load(const tinyxml2::XMLDocument &doc)
             for (const tinyxml2::XMLElement *memorynode = node->FirstChildElement(); memorynode; memorynode = memorynode->NextSiblingElement()) {
                 const std::string memorynodename = memorynode->Name();
                 if (memorynodename == "alloc") {
-                    _alloc[memorynode->GetText()] = allocationId;
-                    const char *init = memorynode->Attribute("init");
-                    if (init && strcmp(init,"false")==0) {
+                    AllocFunc temp;
+                    temp.groupId = allocationId;
+
+                    if (memorynode->Attribute("init", "false"))
                         returnuninitdata.insert(memorynode->GetText());
-                    }
-                } else if (memorynodename == "dealloc")
-                    _dealloc[memorynode->GetText()] = allocationId;
-                else if (memorynodename == "use")
-                    use.insert(memorynode->GetText());
+
+                    const char *arg = memorynode->Attribute("arg");
+                    if (arg)
+                        temp.arg = atoi(arg);
+                    else
+                        temp.arg = -1;
+                    _alloc[memorynode->GetText()] = temp;
+                } else if (memorynodename == "dealloc") {
+                    AllocFunc temp;
+                    temp.groupId = allocationId;
+                    const char *arg = memorynode->Attribute("arg");
+                    if (arg)
+                        temp.arg = atoi(arg);
+                    else
+                        temp.arg = 1;
+                    _dealloc[memorynode->GetText()] = temp;
+                } else if (memorynodename == "use")
+                    functions[memorynode->GetText()].use = true;
                 else
                     unknown_elements.insert(memorynodename);
             }
@@ -177,23 +210,15 @@ Library::Error Library::load(const tinyxml2::XMLDocument &doc)
         }
 
         else if (nodename == "function") {
-            const char *name_char = node->Attribute("name");
-            if (name_char == nullptr)
+            const char *name = node->Attribute("name");
+            if (name == nullptr)
                 return Error(MISSING_ATTRIBUTE, "name");
-            std::string name = name_char;
-            for (;;) {
-                const std::string::size_type pos = name.find(',');
-                if (pos == std::string::npos)
-                    break;
-                const Error &err = loadFunction(node, name.substr(0,pos), unknown_elements);
+            const std::vector<std::string> names(getnames(name));
+            for (unsigned int i = 0U; i < names.size(); ++i) {
+                const Error &err = loadFunction(node, names[i], unknown_elements);
                 if (err.errorcode != ErrorCode::OK)
                     return err;
-                name.erase(0,pos+1);
             }
-
-            const Error &err = loadFunction(node, name, unknown_elements);
-            if (err.errorcode != ErrorCode::OK)
-                return err;
         }
 
         else if (nodename == "reflection") {
@@ -217,10 +242,8 @@ Library::Error Library::load(const tinyxml2::XMLDocument &doc)
                 return Error(MISSING_ATTRIBUTE, "ext");
             _markupExtensions.insert(extension);
 
-            const char * const reporterrors = node->Attribute("reporterrors");
-            _reporterrors[extension] = (reporterrors && strcmp(reporterrors, "true") == 0);
-            const char * const aftercode = node->Attribute("aftercode");
-            _processAfterCode[extension] = (aftercode && strcmp(aftercode, "true") == 0);
+            _reporterrors[extension] = (node->Attribute("reporterrors", "true") != nullptr);
+            _processAfterCode[extension] = (node->Attribute("aftercode", "true") != nullptr);
 
             for (const tinyxml2::XMLElement *markupnode = node->FirstChildElement(); markupnode; markupnode = markupnode->NextSiblingElement()) {
                 const std::string markupnodename = markupnode->Name();
@@ -319,6 +342,12 @@ Library::Error Library::load(const tinyxml2::XMLDocument &doc)
             const char* const endPattern = node->Attribute("endPattern");
             if (endPattern)
                 container.endPattern = endPattern;
+            const char* const itEndPattern = node->Attribute("itEndPattern");
+            if (itEndPattern)
+                container.itEndPattern = itEndPattern;
+            const char* const opLessAllowed = node->Attribute("opLessAllowed");
+            if (opLessAllowed)
+                container.opLessAllowed = std::string(opLessAllowed) == "true";
 
             for (const tinyxml2::XMLElement *containerNode = node->FirstChildElement(); containerNode; containerNode = containerNode->NextSiblingElement()) {
                 const std::string containerNodeName = containerNode->Name();
@@ -347,6 +376,16 @@ Library::Error Library::load(const tinyxml2::XMLDocument &doc)
                                 action = Container::POP;
                             else if (actionName == "find")
                                 action = Container::FIND;
+                            else if (actionName == "insert")
+                                action = Container::INSERT;
+                            else if (actionName == "erase")
+                                action = Container::ERASE;
+                            else if (actionName == "change-content")
+                                action = Container::CHANGE_CONTENT;
+                            else if (actionName == "change-internal")
+                                action = Container::CHANGE_INTERNAL;
+                            else if (actionName == "change")
+                                action = Container::CHANGE;
                             else
                                 return Error(BAD_ATTRIBUTE_VALUE, actionName);
                         }
@@ -367,6 +406,8 @@ Library::Error Library::load(const tinyxml2::XMLDocument &doc)
                                 yield = Container::START_ITERATOR;
                             else if (yieldName == "end-iterator")
                                 yield = Container::END_ITERATOR;
+                            else if (yieldName == "iterator")
+                                yield = Container::ITERATOR;
                             else if (yieldName == "size")
                                 yield = Container::SIZE;
                             else if (yieldName == "empty")
@@ -412,7 +453,9 @@ Library::Error Library::load(const tinyxml2::XMLDocument &doc)
             const char * const sign = node->Attribute("sign");
             if (sign)
                 podType.sign = *sign;
-            podtypes[name] = podType;
+            const std::vector<std::string> names(getnames(name));
+            for (unsigned int i = 0U; i < names.size(); ++i)
+                podtypes[names[i]] = podType;
         }
 
         else if (nodename == "platformtype") {
@@ -448,7 +491,7 @@ Library::Error Library::load(const tinyxml2::XMLDocument &doc)
                     unknown_elements.insert(typenodename);
             }
             if (platform.empty()) {
-                const PlatformType * const type_ptr = platform_type(type_name, "");
+                const PlatformType * const type_ptr = platform_type(type_name, emptyString);
                 if (type_ptr) {
                     if (*type_ptr == type)
                         return Error(DUPLICATE_PLATFORM_TYPE, type_name);
@@ -489,41 +532,50 @@ Library::Error Library::loadFunction(const tinyxml2::XMLElement * const node, co
     if (name.empty())
         return Error(OK);
 
+    Function& func = functions[name];
+
     for (const tinyxml2::XMLElement *functionnode = node->FirstChildElement(); functionnode; functionnode = functionnode->NextSiblingElement()) {
         const std::string functionnodename = functionnode->Name();
         if (functionnodename == "noreturn")
             _noreturn[name] = (strcmp(functionnode->GetText(), "true") == 0);
         else if (functionnodename == "pure")
-            functionpure.insert(name);
+            func.ispure = true;
         else if (functionnodename == "const") {
-            functionconst.insert(name);
-            functionpure.insert(name); // a constant function is pure
+            func.ispure = true;
+            func.isconst = true; // a constant function is pure
         } else if (functionnodename == "leak-ignore")
-            leakignore.insert(name);
+            func.leakignore = true;
         else if (functionnodename == "use-retval")
-            _useretval.insert(name);
-        else if (functionnodename == "arg" && functionnode->Attribute("nr") != nullptr) {
-            const bool bAnyArg = strcmp(functionnode->Attribute("nr"),"any")==0;
-            const int nr = (bAnyArg) ? -1 : atoi(functionnode->Attribute("nr"));
-            bool notbool = false;
-            bool notnull = false;
-            bool notuninit = false;
-            bool formatstr = false;
-            bool strz = false;
-            std::string& valid = argumentChecks[name][nr].valid;
-            std::list<ArgumentChecks::MinSize>& minsizes = argumentChecks[name][nr].minsizes;
+            func.useretval = true;
+        else if (functionnodename == "returnValue") {
+            if (const char *expr = functionnode->GetText())
+                _returnValue[name] = expr;
+            if (const char *type = functionnode->Attribute("type"))
+                _returnValueType[name] = type;
+            if (const char *container = functionnode->Attribute("container"))
+                _returnValueContainer[name] = std::atoi(container);
+        } else if (functionnodename == "arg") {
+            const char* argNrString = functionnode->Attribute("nr");
+            if (!argNrString)
+                return Error(MISSING_ATTRIBUTE, "nr");
+            const bool bAnyArg = strcmp(argNrString, "any") == 0;
+            const bool bVariadicArg = strcmp(argNrString, "variadic") == 0;
+            const int nr = (bAnyArg || bVariadicArg) ? -1 : std::atoi(argNrString);
+            ArgumentChecks &ac = func.argumentChecks[nr];
+            ac.optional  = functionnode->Attribute("default") != nullptr;
+            ac.variadic = bVariadicArg;
             for (const tinyxml2::XMLElement *argnode = functionnode->FirstChildElement(); argnode; argnode = argnode->NextSiblingElement()) {
                 const std::string argnodename = argnode->Name();
                 if (argnodename == "not-bool")
-                    notbool = true;
+                    ac.notbool = true;
                 else if (argnodename == "not-null")
-                    notnull = true;
+                    ac.notnull = true;
                 else if (argnodename == "not-uninit")
-                    notuninit = true;
+                    ac.notuninit = true;
                 else if (argnodename == "formatstr")
-                    formatstr = true;
+                    ac.formatstr = true;
                 else if (argnodename == "strz")
-                    strz = true;
+                    ac.strz = true;
                 else if (argnodename == "valid") {
                     // Validate the validation expression
                     const char *p = argnode->GetText();
@@ -547,7 +599,7 @@ Library::Error Library::loadFunction(const tinyxml2::XMLElement * const node, co
                         return Error(BAD_ATTRIBUTE_VALUE, argnode->GetText());
 
                     // Set validation expression
-                    valid = argnode->GetText();
+                    ac.valid = argnode->GetText();
                 }
 
                 else if (argnodename == "minsize") {
@@ -573,31 +625,78 @@ Library::Error Library::loadFunction(const tinyxml2::XMLElement * const node, co
                     if (strlen(argattr) != 1 || argattr[0]<'0' || argattr[0]>'9')
                         return Error(BAD_ATTRIBUTE_VALUE, argattr);
 
-                    minsizes.push_back(ArgumentChecks::MinSize(type,argattr[0]-'0'));
+                    ac.minsizes.reserve(type == ArgumentChecks::MinSize::MUL ? 2 : 1);
+                    ac.minsizes.push_back(ArgumentChecks::MinSize(type,argattr[0]-'0'));
                     if (type == ArgumentChecks::MinSize::MUL) {
-                        const char *arg2attr  = argnode->Attribute("arg2");
+                        const char *arg2attr = argnode->Attribute("arg2");
                         if (!arg2attr)
                             return Error(MISSING_ATTRIBUTE, "arg2");
                         if (strlen(arg2attr) != 1 || arg2attr[0]<'0' || arg2attr[0]>'9')
                             return Error(BAD_ATTRIBUTE_VALUE, arg2attr);
-                        minsizes.back().arg2 = arg2attr[0] - '0';
+                        ac.minsizes.back().arg2 = arg2attr[0] - '0';
                     }
                 }
 
+                else if (argnodename == "iterator") {
+                    ac.iteratorInfo.it = true;
+                    const char* str = argnode->Attribute("type");
+                    ac.iteratorInfo.first = str ? (std::strcmp(str, "first") == 0) : false;
+                    ac.iteratorInfo.last = str ? (std::strcmp(str, "last") == 0) : false;
+                    str = argnode->Attribute("container");
+                    ac.iteratorInfo.container = str ? std::atoi(str) : 0;
+                }
+
                 else
                     unknown_elements.insert(argnodename);
             }
-            argumentChecks[name][nr].notbool   = notbool;
-            argumentChecks[name][nr].notnull   = notnull;
-            argumentChecks[name][nr].notuninit = notuninit;
-            argumentChecks[name][nr].formatstr = formatstr;
-            argumentChecks[name][nr].strz      = strz;
         } else if (functionnodename == "ignorefunction") {
-            _ignorefunction.insert(name);
+            func.ignore = true;
         } else if (functionnodename == "formatstr") {
+            func.formatstr = true;
             const tinyxml2::XMLAttribute* scan = functionnode->FindAttribute("scan");
             const tinyxml2::XMLAttribute* secure = functionnode->FindAttribute("secure");
-            _formatstr[name] = std::make_pair(scan && scan->BoolValue(), secure && secure->BoolValue());
+            func.formatstr_scan = scan && scan->BoolValue();
+            func.formatstr_secure = secure && secure->BoolValue();
+        } else if (functionnodename == "warn") {
+            WarnInfo wi;
+            const char* const severity = functionnode->Attribute("severity");
+            if (severity == nullptr)
+                return Error(MISSING_ATTRIBUTE, "severity");
+            wi.severity = Severity::fromString(severity);
+
+            const char* const cstd = functionnode->Attribute("cstd");
+            if (cstd) {
+                if (!wi.standards.setC(cstd))
+                    return Error(BAD_ATTRIBUTE_VALUE, cstd);
+            } else
+                wi.standards.c = Standards::C89;
+
+            const char* const cppstd = functionnode->Attribute("cppstd");
+            if (cppstd) {
+                if (!wi.standards.setCPP(cppstd))
+                    return Error(BAD_ATTRIBUTE_VALUE, cppstd);
+            } else
+                wi.standards.cpp = Standards::CPP03;
+
+            const char* const reason = functionnode->Attribute("reason");
+            const char* const alternatives = functionnode->Attribute("alternatives");
+            if (reason && alternatives) {
+                // Construct message
+                wi.message = std::string(reason) + " function '" + name + "' called. It is recommended to use ";
+                std::vector<std::string> alt = getnames(alternatives);
+                for (std::size_t i = 0; i < alt.size(); ++i) {
+                    wi.message += "'" + alt[i] + "'";
+                    if (i == alt.size() - 1)
+                        wi.message += " instead.";
+                    else if (i == alt.size() - 2)
+                        wi.message += " or ";
+                    else
+                        wi.message += ", ";
+                }
+            } else
+                wi.message = functionnode->GetText();
+
+            functionwarn[name] = wi;
         } else
             unknown_elements.insert(functionnodename);
     }
@@ -609,7 +708,7 @@ bool Library::isargvalid(const Token *ftok, int argnr, const MathLib::bigint arg
     const ArgumentChecks *ac = getarg(ftok, argnr);
     if (!ac || ac->valid.empty())
         return true;
-    TokenList tokenList(0);
+    TokenList tokenList(nullptr);
     std::istringstream istr(ac->valid + ',');
     tokenList.createTokens(istr);
     for (Token *tok = tokenList.front(); tok; tok = tok->next()) {
@@ -631,18 +730,30 @@ bool Library::isargvalid(const Token *ftok, int argnr, const MathLib::bigint arg
     return false;
 }
 
-static std::string functionName(const Token *ftok, bool *error)
+std::string Library::getFunctionName(const Token *ftok, bool *error) const
 {
     if (!ftok) {
         *error = true;
         return "";
     }
-    if (ftok->isName())
+    if (ftok->isName()) {
+        for (const Scope *scope = ftok->scope(); scope; scope = scope->nestedIn) {
+            if (!scope->isClassOrStruct())
+                continue;
+            const std::vector<Type::BaseInfo> &derivedFrom = scope->definedType->derivedFrom;
+            for (unsigned int i = 0; i < derivedFrom.size(); ++i) {
+                const Type::BaseInfo &baseInfo = derivedFrom[i];
+                const std::string name(baseInfo.name + "::" + ftok->str());
+                if (functions.find(name) != functions.end() && matchArguments(ftok, name))
+                    return name;
+            }
+        }
         return ftok->str();
+    }
     if (ftok->str() == "::") {
         if (!ftok->astOperand2())
-            return functionName(ftok->astOperand1(), error);
-        return functionName(ftok->astOperand1(),error) + "::" + functionName(ftok->astOperand2(),error);
+            return getFunctionName(ftok->astOperand1(), error);
+        return getFunctionName(ftok->astOperand1(),error) + "::" + getFunctionName(ftok->astOperand2(),error);
     }
     if (ftok->str() == "." && ftok->astOperand1()) {
         const std::string type = astCanonicalType(ftok->astOperand1());
@@ -651,13 +762,13 @@ static std::string functionName(const Token *ftok, bool *error)
             return "";
         }
 
-        return type + "::" + functionName(ftok->astOperand2(),error);
+        return type + "::" + getFunctionName(ftok->astOperand2(),error);
     }
     *error = true;
     return "";
 }
 
-static std::string functionName(const Token *ftok)
+std::string Library::getFunctionName(const Token *ftok) const
 {
     if (!Token::Match(ftok, "%name% ("))
         return "";
@@ -665,7 +776,7 @@ static std::string functionName(const Token *ftok)
     // Lookup function name using AST..
     if (ftok->astParent()) {
         bool error = false;
-        std::string ret = functionName(ftok->next()->astOperand1(), &error);
+        std::string ret = getFunctionName(ftok->next()->astOperand1(), &error);
         return error ? std::string() : ret;
     }
 
@@ -688,9 +799,9 @@ bool Library::isnullargbad(const Token *ftok, int argnr) const
     const ArgumentChecks *arg = getarg(ftok, argnr);
     if (!arg) {
         // scan format string argument should not be null
-        const std::string funcname = functionName(ftok);
-        std::map<std::string, std::pair<bool, bool> >::const_iterator it = _formatstr.find(funcname);
-        if (it != _formatstr.end() && it->second.first)
+        const std::string funcname = getFunctionName(ftok);
+        std::map<std::string, Function>::const_iterator it = functions.find(funcname);
+        if (it != functions.cend() && it->second.formatstr && it->second.formatstr_scan)
             return true;
     }
     return arg && arg->notnull;
@@ -701,27 +812,41 @@ bool Library::isuninitargbad(const Token *ftok, int argnr) const
     const ArgumentChecks *arg = getarg(ftok, argnr);
     if (!arg) {
         // non-scan format string argument should not be uninitialized
-        const std::string funcname = functionName(ftok);
-        std::map<std::string, std::pair<bool, bool> >::const_iterator it = _formatstr.find(funcname);
-        if (it != _formatstr.end() && !it->second.first)
+        const std::string funcname = getFunctionName(ftok);
+        std::map<std::string, Function>::const_iterator it = functions.find(funcname);
+        if (it != functions.cend() && it->second.formatstr && !it->second.formatstr_scan)
             return true;
     }
     return arg && arg->notuninit;
 }
 
 
-/** get allocation id for function */
-int Library::alloc(const Token *tok) const
+/** get allocation info for function */
+const Library::AllocFunc* Library::alloc(const Token *tok) const
 {
-    const std::string funcname = functionName(tok);
-    return isNotLibraryFunction(tok) && argumentChecks.find(funcname) != argumentChecks.end() ? 0 : getid(_alloc, funcname);
+    const std::string funcname = getFunctionName(tok);
+    return isNotLibraryFunction(tok) && functions.find(funcname) != functions.end() ? nullptr : getAllocDealloc(_alloc, funcname);
+}
+
+/** get deallocation info for function */
+const Library::AllocFunc* Library::dealloc(const Token *tok) const
+{
+    const std::string funcname = getFunctionName(tok);
+    return isNotLibraryFunction(tok) && functions.find(funcname) != functions.end() ? nullptr : getAllocDealloc(_dealloc, funcname);
+}
+
+/** get allocation id for function */
+int Library::alloc(const Token *tok, int arg) const
+{
+    const Library::AllocFunc* af = alloc(tok);
+    return (af && af->arg == arg) ? af->groupId : 0;
 }
 
 /** get deallocation id for function */
-int Library::dealloc(const Token *tok) const
+int Library::dealloc(const Token *tok, int arg) const
 {
-    const std::string funcname = functionName(tok);
-    return isNotLibraryFunction(tok) && argumentChecks.find(funcname) != argumentChecks.end() ? 0 : getid(_dealloc, funcname);
+    const Library::AllocFunc* af = dealloc(tok);
+    return (af && af->arg == arg) ? af->groupId : 0;
 }
 
 
@@ -729,15 +854,14 @@ const Library::ArgumentChecks * Library::getarg(const Token *ftok, int argnr) co
 {
     if (isNotLibraryFunction(ftok))
         return nullptr;
-    std::map<std::string, std::map<int, ArgumentChecks> >::const_iterator it1;
-    it1 = argumentChecks.find(functionName(ftok));
-    if (it1 == argumentChecks.end())
+    std::map<std::string, Function>::const_iterator it1 = functions.find(getFunctionName(ftok));
+    if (it1 == functions.cend())
         return nullptr;
-    const std::map<int,ArgumentChecks>::const_iterator it2 = it1->second.find(argnr);
-    if (it2 != it1->second.end())
+    const std::map<int,ArgumentChecks>::const_iterator it2 = it1->second.argumentChecks.find(argnr);
+    if (it2 != it1->second.argumentChecks.cend())
         return &it2->second;
-    const std::map<int,ArgumentChecks>::const_iterator it3 = it1->second.find(-1);
-    if (it3 != it1->second.end())
+    const std::map<int,ArgumentChecks>::const_iterator it3 = it1->second.argumentChecks.find(-1);
+    if (it3 != it1->second.argumentChecks.cend())
         return &it3->second;
     return nullptr;
 }
@@ -773,7 +897,7 @@ bool Library::isScopeNoReturn(const Token *end, std::string *unknownFunc) const
     return false;
 }
 
-const Library::Container* Library::detectContainer(const Token* typeStart) const
+const Library::Container* Library::detectContainer(const Token* typeStart, bool iterator) const
 {
     for (std::map<std::string, Container>::const_iterator i = containers.begin(); i != containers.end(); ++i) {
         const Container& container = i->second;
@@ -781,12 +905,13 @@ const Library::Container* Library::detectContainer(const Token* typeStart) const
             continue;
 
         if (Token::Match(typeStart, container.startPattern.c_str())) {
-            if (container.endPattern.empty())
+            if (!iterator && container.endPattern.empty()) // If endPattern is undefined, it will always match, but itEndPattern has to be defined.
                 return &container;
 
             for (const Token* tok = typeStart; tok && !tok->varId(); tok = tok->next()) {
                 if (tok->link()) {
-                    if (Token::Match(tok->link(), container.endPattern.c_str()))
+                    const std::string& endPattern = iterator ? container.itEndPattern : container.endPattern;
+                    if (Token::Match(tok->link(), endPattern.c_str()))
                         return &container;
                     break;
                 }
@@ -796,6 +921,25 @@ const Library::Container* Library::detectContainer(const Token* typeStart) const
     return nullptr;
 }
 
+bool Library::isContainerYield(const Token * const cond, Library::Container::Yield y, const std::string& fallback)
+{
+    if (!cond)
+        return false;
+    if (cond->str() == "(") {
+        const Token* tok = cond->astOperand1();
+        if (tok && tok->str() == ".") {
+            if (tok->astOperand1() && tok->astOperand1()->valueType()) {
+                if (const Library::Container *container = tok->astOperand1()->valueType()->container) {
+                    return tok->astOperand2() && y == container->getYield(tok->astOperand2()->str());
+                }
+            } else if (!fallback.empty()) {
+                return Token::simpleMatch(cond, "( )") && cond->previous()->str() == fallback;
+            }
+        }
+    }
+    return false;
+}
+
 // returns true if ftok is not a library function
 bool Library::isNotLibraryFunction(const Token *ftok) const
 {
@@ -806,41 +950,161 @@ bool Library::isNotLibraryFunction(const Token *ftok) const
     if (ftok->varId())
         return true;
 
-    int callargs = 0;
-    for (const Token *tok = ftok->tokAt(2); tok && tok->str() != ")"; tok = tok->next()) {
-        if (callargs == 0)
-            callargs = 1;
-        if (tok->str() == ",")
-            callargs++;
-        else if (tok->link() && Token::Match(tok, "<|(|["))
-            tok = tok->link();
-    }
-    const std::map<std::string, std::map<int, ArgumentChecks> >::const_iterator it = argumentChecks.find(functionName(ftok));
-    if (it == argumentChecks.end())
-        return (callargs != 0);
+    return !matchArguments(ftok, getFunctionName(ftok));
+}
+
+bool Library::matchArguments(const Token *ftok, const std::string &functionName) const
+{
+    int callargs = numberOfArguments(ftok);
+    const std::map<std::string, Function>::const_iterator it = functions.find(functionName);
+    if (it == functions.cend())
+        return (callargs == 0);
     int args = 0;
-    for (std::map<int, ArgumentChecks>::const_iterator it2 = it->second.begin(); it2 != it->second.end(); ++it2) {
+    int firstOptionalArg = -1;
+    for (std::map<int, ArgumentChecks>::const_iterator it2 = it->second.argumentChecks.cbegin(); it2 != it->second.argumentChecks.cend(); ++it2) {
         if (it2->first > args)
             args = it2->first;
-        if (it2->second.formatstr)
-            return args > callargs;
+        if (it2->second.optional && (firstOptionalArg == -1 || firstOptionalArg > it2->first))
+            firstOptionalArg = it2->first;
+
+        if (it2->second.formatstr || it2->second.variadic)
+            return args <= callargs;
     }
-    return args != callargs;
+    return (firstOptionalArg < 0) ? args == callargs : (callargs >= firstOptionalArg-1 && callargs <= args);
+}
+
+const Library::WarnInfo* Library::getWarnInfo(const Token* ftok) const
+{
+    if (isNotLibraryFunction(ftok))
+        return nullptr;
+    std::map<std::string, WarnInfo>::const_iterator i = functionwarn.find(getFunctionName(ftok));
+    if (i == functionwarn.cend())
+        return nullptr;
+    return &i->second;
+}
+
+bool Library::formatstr_function(const Token* ftok) const
+{
+    if (isNotLibraryFunction(ftok))
+        return false;
+
+    std::map<std::string, Function>::const_iterator it = functions.find(getFunctionName(ftok));
+    if (it != functions.cend())
+        return it->second.formatstr;
+    return false;
+}
+
+int Library::formatstr_argno(const Token* ftok) const
+{
+    const std::map<int, Library::ArgumentChecks>& argumentChecksFunc = functions.at(getFunctionName(ftok)).argumentChecks;
+    for (std::map<int, Library::ArgumentChecks>::const_iterator i = argumentChecksFunc.cbegin(); i != argumentChecksFunc.cend(); ++i) {
+        if (i->second.formatstr) {
+            return i->first - 1;
+        }
+    }
+    return -1;
+}
+
+bool Library::formatstr_scan(const Token* ftok) const
+{
+    return functions.at(getFunctionName(ftok)).formatstr_scan;
+}
+
+bool Library::formatstr_secure(const Token* ftok) const
+{
+    return functions.at(getFunctionName(ftok)).formatstr_secure;
 }
 
 bool Library::isUseRetVal(const Token* ftok) const
 {
-    return (!isNotLibraryFunction(ftok) &&
-            _useretval.find(functionName(ftok)) != _useretval.end());
+    if (isNotLibraryFunction(ftok))
+        return false;
+    std::map<std::string, Function>::const_iterator it = functions.find(getFunctionName(ftok));
+    if (it != functions.cend())
+        return it->second.useretval;
+    return false;
 }
 
+const std::string& Library::returnValue(const Token *ftok) const
+{
+    if (isNotLibraryFunction(ftok))
+        return emptyString;
+    std::map<std::string, std::string>::const_iterator it = _returnValue.find(getFunctionName(ftok));
+    return it != _returnValue.end() ? it->second : emptyString;
+}
+
+const std::string& Library::returnValueType(const Token *ftok) const
+{
+    if (isNotLibraryFunction(ftok))
+        return emptyString;
+    std::map<std::string, std::string>::const_iterator it = _returnValueType.find(getFunctionName(ftok));
+    return it != _returnValueType.end() ? it->second : emptyString;
+}
+
+int Library::returnValueContainer(const Token *ftok) const
+{
+    if (isNotLibraryFunction(ftok))
+        return -1;
+    std::map<std::string, int>::const_iterator it = _returnValueContainer.find(getFunctionName(ftok));
+    return it != _returnValueContainer.end() ? it->second : -1;
+}
+
+bool Library::hasminsize(const std::string &functionName) const
+{
+    std::map<std::string, Function>::const_iterator it1 = functions.find(functionName);
+    if (it1 == functions.cend())
+        return false;
+    for (std::map<int, ArgumentChecks>::const_iterator it2 = it1->second.argumentChecks.cbegin(); it2 != it1->second.argumentChecks.cend(); ++it2) {
+        if (!it2->second.minsizes.empty())
+            return true;
+    }
+    return false;
+}
+
+bool Library::ignorefunction(const std::string& functionName) const
+{
+    std::map<std::string, Function>::const_iterator it = functions.find(functionName);
+    if (it != functions.cend())
+        return it->second.ignore;
+    return false;
+}
+bool Library::isUse(const std::string& functionName) const
+{
+    std::map<std::string, Function>::const_iterator it = functions.find(functionName);
+    if (it != functions.cend())
+        return it->second.use;
+    return false;
+}
+bool Library::isLeakIgnore(const std::string& functionName) const
+{
+    std::map<std::string, Function>::const_iterator it = functions.find(functionName);
+    if (it != functions.cend())
+        return it->second.leakignore;
+    return false;
+}
+bool Library::isFunctionConst(const std::string& functionName, bool pure) const
+{
+    std::map<std::string, Function>::const_iterator it = functions.find(functionName);
+    if (it != functions.cend())
+        return pure ? it->second.ispure : it->second.isconst;
+    return false;
+}
+bool Library::isFunctionConst(const Token *ftok) const
+{
+    if (ftok->function() && ftok->function()->isAttributeConst())
+        return true;
+    if (isNotLibraryFunction(ftok))
+        return false;
+    std::map<std::string, Function>::const_iterator it = functions.find(getFunctionName(ftok));
+    return (it != functions.end() && it->second.isconst);
+}
 bool Library::isnoreturn(const Token *ftok) const
 {
     if (ftok->function() && ftok->function()->isAttributeNoreturn())
         return true;
     if (isNotLibraryFunction(ftok))
         return false;
-    std::map<std::string, bool>::const_iterator it = _noreturn.find(functionName(ftok));
+    std::map<std::string, bool>::const_iterator it = _noreturn.find(getFunctionName(ftok));
     return (it != _noreturn.end() && it->second);
 }
 
@@ -850,7 +1114,7 @@ bool Library::isnotnoreturn(const Token *ftok) const
         return false;
     if (isNotLibraryFunction(ftok))
         return false;
-    std::map<std::string, bool>::const_iterator it = _noreturn.find(functionName(ftok));
+    std::map<std::string, bool>::const_iterator it = _noreturn.find(getFunctionName(ftok));
     return (it != _noreturn.end() && !it->second);
 }
 
