@@ -48,6 +48,24 @@ void bufferAccessOutOfBounds(void)
     // cppcheck-suppress bufferAccessOutOfBounds
     // cppcheck-suppress redundantCopy
     strncpy(a,"abcde",6);
+    // cppcheck-suppress bufferAccessOutOfBounds
+    // cppcheck-suppress redundantCopy
+    strncpy(a,"a",6);
+    // cppcheck-suppress redundantCopy
+    strncpy(a,"abcdefgh",4);
+    // valid call
+    strncpy_s(a,5,"abcd",5);
+    // string will be truncated, error is returned, but no buffer overflow
+    strncpy_s(a,5,"abcde",6);
+    // cppcheck-suppress bufferAccessOutOfBounds
+    strncpy_s(a,5,"a",6);
+    strncpy_s(a,5,"abcdefgh",4);
+    // valid call
+    strncat_s(a,5,"1",2);
+    // cppcheck-suppress bufferAccessOutOfBounds
+    strncat_s(a,10,"1",2);
+    // cppcheck-suppress bufferAccessOutOfBounds
+    strncat_s(a,5,"1",5);
     fread(a,1,5,stdin);
     // cppcheck-suppress bufferAccessOutOfBounds
     fread(a,1,6,stdin);
@@ -71,21 +89,26 @@ void nullpointer(int value)
 {
     int res = 0;
     FILE *fp;
+    wchar_t *pWcsUninit;
 
+#ifndef __CYGWIN__
     // cppcheck-suppress nullPointer
     clearerr(0);
     // cppcheck-suppress ignoredReturnValue
     // cppcheck-suppress nullPointer
     feof(0);
+#endif
     // cppcheck-suppress nullPointer
-    fgetc(0);
+    (void)fgetc(0);
     // cppcheck-suppress nullPointer
     fclose(0);
+#ifndef __CYGWIN__
     // cppcheck-suppress ignoredReturnValue
     // cppcheck-suppress nullPointer
     ferror(0);
+#endif
     // cppcheck-suppress nullPointer
-    ftell(0);
+    (void)ftell(0);
     // cppcheck-suppress nullPointer
     puts(0);
     // cppcheck-suppress nullPointer
@@ -117,50 +140,99 @@ void nullpointer(int value)
     strchr(0,0);
     // cppcheck-suppress ignoredReturnValue
     // cppcheck-suppress nullPointer
+    wcschr(0,0);
+    // cppcheck-suppress ignoredReturnValue
+    // cppcheck-suppress nullPointer
     strlen(0);
+    // cppcheck-suppress ignoredReturnValue
+    // cppcheck-suppress nullPointer
+    wcslen(0);
     // cppcheck-suppress nullPointer
     strcpy(0,0);
+    // cppcheck-suppress nullPointer
+    wcscpy(0,0);
     // cppcheck-suppress ignoredReturnValue
     // cppcheck-suppress nullPointer
     strspn(0,0);
     // cppcheck-suppress ignoredReturnValue
     // cppcheck-suppress nullPointer
+    wcsspn(0,0);
+    // cppcheck-suppress ignoredReturnValue
+    // cppcheck-suppress nullPointer
     strcspn(0,0);
     // cppcheck-suppress ignoredReturnValue
     // cppcheck-suppress nullPointer
+    wcscspn(0,0);
+    // cppcheck-suppress ignoredReturnValue
+    // cppcheck-suppress nullPointer
     strcoll(0,0);
+    // cppcheck-suppress ignoredReturnValue
+    // cppcheck-suppress nullPointer
+    wcscoll(0,0);
     // cppcheck-suppress nullPointer
     strcat(0,0);
+    // cppcheck-suppress nullPointer
+    wcscat(0,0);
     // cppcheck-suppress ignoredReturnValue
     // cppcheck-suppress nullPointer
     strcmp(0,0);
+    // cppcheck-suppress ignoredReturnValue
+    // cppcheck-suppress nullPointer
+    wcscmp(0,0);
     // cppcheck-suppress nullPointer
     strncpy(0,0,1);
     // cppcheck-suppress nullPointer
+    strncpy_s(0,1,1,1);
+    // cppcheck-suppress nullPointer
+    strncpy_s(1,1,0,1);
+    // cppcheck-suppress nullPointer
+    wcsncpy(0,0,1);
+    // cppcheck-suppress nullPointer
     strncat(0,0,1);
+    // cppcheck-suppress nullPointer
+    strncat_s(0,1,1,1);
+    // cppcheck-suppress nullPointer
+    strncat_s(1,1,0,1);
+    // cppcheck-suppress nullPointer
+    wcsncat(0,0,1);
     // cppcheck-suppress ignoredReturnValue
     // cppcheck-suppress nullPointer
     strncmp(0,0,1);
     // cppcheck-suppress ignoredReturnValue
     // cppcheck-suppress nullPointer
+    wcsncmp(0,0,1);
+    // cppcheck-suppress ignoredReturnValue
+    // cppcheck-suppress nullPointer
     strstr(0,0);
+    // cppcheck-suppress ignoredReturnValue
+    // cppcheck-suppress nullPointer
+    wcsstr(0,0);
     // cppcheck-suppress nullPointer
     strtoul(0,0,0);
     // cppcheck-suppress nullPointer
+    wcstoul(0,0,0);
+    // cppcheck-suppress nullPointer
     strtoull(0,0,0);
     // cppcheck-suppress nullPointer
+    wcstoull(0,0,0);
+    // cppcheck-suppress nullPointer
     strtol(0,0,0);
+    // cppcheck-suppress nullPointer
+    wcstol(0,0,0);
 
     // #6100 False positive nullPointer - calling mbstowcs(NULL,)
     res += mbstowcs(0,"",0);
-    // cppcheck-suppress unreadVariable
     res += wcstombs(0,L"",0);
 
     strtok(NULL,"xyz");
+    wcstok(NULL,L"xyz",&pWcsUninit);
 
     strxfrm(0,"foo",0);
     // TODO: error message (#6306 and http://trac.cppcheck.net/changeset/d11eb4931aea51cf2cb74faccdcd2a3289b818d6/)
     strxfrm(0,"foo",42);
+    wcsxfrm(0,L"foo",0);
+    // TODO: error message when arg1==NULL and arg3!=0 #6306: https://trac.cppcheck.net/ticket/6306#comment:2
+    wcsxfrm(0,L"foo",42);
 
     snprintf(NULL, 0, "someformatstring"); // legal
     // cppcheck-suppress nullPointer
@@ -169,6 +241,9 @@ void nullpointer(int value)
     scanf("%i", &res);
     // cppcheck-suppress nullPointer
     scanf("%i", NULL);
+    wscanf(L"%i", &res);
+    // cppcheck-suppress nullPointer
+    wscanf(L"%i", NULL);
 }
 
 void nullpointerMemchr1(char *p, char *s)
@@ -265,7 +340,7 @@ void uninit_fgetc(void)
 {
     FILE *fp;
     // cppcheck-suppress uninitvar
-    fgetc(fp);
+    (void)fgetc(fp);
 }
 
 void uninit_fgetpos(void)
@@ -343,7 +418,7 @@ void uninit_ftell(void)
 {
     FILE *fp;
     // cppcheck-suppress uninitvar
-    ftell(fp);
+    (void)ftell(fp);
 }
 
 void uninit_puts(void)
@@ -424,18 +499,17 @@ void uninitvar_asctime(void)
 {
     const struct tm *tm;
     // cppcheck-suppress uninitvar
-    // cppcheck-suppress obsoleteFunctionsasctime
+    // cppcheck-suppress asctimeCalled
     (void)asctime(tm);
 }
 
-#if 0
 void uninitvar_assert(void)
 {
     int i;
+    // cppcheck-suppress checkLibraryNoReturn
     // cppcheck-suppress uninitvar
     assert(i);
 }
-#endif
 
 void uninitvar_sqrt(void)
 {
@@ -1197,17 +1271,14 @@ void uninitvar_floor(void)
 
 void uninitvar_fma(void)
 {
-    // cppcheck-suppress unassignedVariable
     float f1,f2,f3;
     // cppcheck-suppress uninitvar
     (void)fmaf(f1,f2,f3);
 
-    // cppcheck-suppress unassignedVariable
     double d1,d2,d3;
     // cppcheck-suppress uninitvar
     (void)fma(d1,d2,d3);
 
-    // cppcheck-suppress unassignedVariable
     long double ld1,ld2,ld3;
     // cppcheck-suppress uninitvar
     (void)fmal(ld1,ld2,ld3);
@@ -1636,7 +1707,7 @@ void uninitvar_getenv(void)
 void uninitvar_gets(void)
 {
     char *buffer;
-    // cppcheck-suppress obsoleteFunctionsgets
+    // cppcheck-suppress getsCalled
     // cppcheck-suppress uninitvar
     (void)gets(buffer);
 }
@@ -2394,7 +2465,7 @@ void uninitvar_malloc(void)
 void uninitvar_alloca(void)
 {
     size_t size;
-    // cppcheck-suppress obsoleteFunctionsalloca
+    // cppcheck-suppress allocaCalled
     // cppcheck-suppress uninitvar
     (void)alloca(size);
 }
@@ -2972,6 +3043,28 @@ void uninivar_strncpy(void)
     (void)strncpy(s,ct,n);
 }
 
+void uninivar_strncpy_s(char *Ct, size_t N1, char *S, size_t N2)
+{
+    char *ct;
+    char *s;
+    size_t n1;
+    size_t n2;
+
+    // cppcheck-suppress uninitvar
+    (void)strncpy_s(ct,n1,s,n2);
+    // cppcheck-suppress uninitvar
+    (void)strncpy_s(ct,N1,S,N2);
+    // cppcheck-suppress uninitvar
+    (void)strncpy_s(Ct,n1,S,N2);
+    // cppcheck-suppress uninitvar
+    (void)strncpy_s(Ct,N1,s,N2);
+    // cppcheck-suppress uninitvar
+    (void)strncpy_s(Ct,N1,S,n2);
+
+    // no warning is expected for
+    (void)strncpy_s(Ct,N1,S,N2);
+}
+
 void uninivar_strpbrk(void)
 {
     char *cs;
@@ -2998,6 +3091,28 @@ void uninivar_strncat(char *Ct, char *S, size_t N)
     (void)strncat(Ct,S,N);
 }
 
+void uninivar_strncat_s(char *Ct, size_t N1, char *S, size_t N2)
+{
+    char *ct;
+    char *s;
+    size_t n1;
+    size_t n2;
+
+    // cppcheck-suppress uninitvar
+    (void)strncat_s(ct,n1,s,n2);
+    // cppcheck-suppress uninitvar
+    (void)strncat_s(ct,N1,S,N2);
+    // cppcheck-suppress uninitvar
+    (void)strncat_s(Ct,n1,S,N2);
+    // cppcheck-suppress uninitvar
+    (void)strncat_s(Ct,N1,s,N2);
+    // cppcheck-suppress uninitvar
+    (void)strncat_s(Ct,N1,S,n2);
+
+    // no warning is expected for
+    (void)strncat_s(Ct,N1,S,N2);
+}
+
 void uninivar_wcsncat(wchar_t *Ct, wchar_t *S, size_t N)
 {
     wchar_t *ct;
@@ -3174,14 +3289,6 @@ void uninivar_strrchr(void)
     (void)strrchr(str,c);
 }
 
-void uninivar_strbprk(void)
-{
-    char *cs;
-    char *ct;
-    // cppcheck-suppress uninitvar
-    (void)strbprk(cs,ct);
-}
-
 void uninivar_wcsrchr(void)
 {
     wchar_t* ws;
@@ -3355,17 +3462,14 @@ void uninitvar_sprintf(char *S, char *Format, int Argument)
     int argument;
     // cppcheck-suppress uninitvar
     (void)sprintf(s,format,argument);
-    // cppcheck-suppress redundantCopy
     // cppcheck-suppress uninitvar
     (void)sprintf(s,Format,Argument);
     // cppcheck-suppress uninitvar
     (void)sprintf(S,format,Argument);
-    // cppcheck-suppress redundantCopy
     // cppcheck-suppress uninitvar
     (void)sprintf(S,Format,argument);
 
     // no warning is expected for
-    // cppcheck-suppress redundantCopy
     (void)sprintf(S,Format,Argument);
 }
 
@@ -3418,18 +3522,14 @@ void uninivar_snprintf(char *S, size_t N, char *Format, int Int)
     (void)snprintf(s,n,format,i);
     // cppcheck-suppress uninitvar
     (void)snprintf(S,n,Format,Int); // n is uninitialized
-    // cppcheck-suppress redundantCopy
     // cppcheck-suppress uninitvar
     (void)snprintf(S,N,format,Int); // format is uninitialized
-    // cppcheck-suppress redundantCopy
     // cppcheck-suppress uninitvar
     (void)snprintf(S,N,Format,i); // i is uninitialized
-    // cppcheck-suppress redundantCopy
-    // cppcheck-suppress uninitvar
+    // TODO cppcheck-suppress uninitvar
     (void)snprintf(s,N,Format,Int);
 
     // no warning is expected for
-    // cppcheck-suppress redundantCopy
     (void)snprintf(S,N,Format,Int);
 }
 
@@ -3578,17 +3678,17 @@ void ignoredReturnValue_abs(int i)
 {
     // cppcheck-suppress ignoredReturnValue
     abs(i);
-    // cppcheck-suppress constStatement
+    // cppcheck-suppress ignoredReturnValue
     abs(-100);
 }
 
 void nullPointer_asctime(void)
 {
     struct tm *tm = 0;
-    // cppcheck-suppress obsoleteFunctionsasctime
+    // cppcheck-suppress asctimeCalled
     // cppcheck-suppress nullPointer
     (void)asctime(tm);
-    // cppcheck-suppress obsoleteFunctionsasctime
+    // cppcheck-suppress asctimeCalled
     // cppcheck-suppress nullPointer
     (void)asctime(0);
 }
@@ -3681,6 +3781,57 @@ void invalidPrintfArgType_printf(void)
 
     // #7016
     uint8_t n = 7;
-    // cppcheck-suppress invalidPrintfArgType_uint
+    // TODO cppcheck-suppress invalidPrintfArgType_uint
     printf("%"PRIi16"\n", n);
 }
+
+
+#define AssertAlwaysTrue(C)  if (C) {}
+
+void valueFlow(void)
+{
+    const char abc[] = "abc";
+    const int three = 3, minusThree = -3;
+    const int c0='0', ca='a', blank=' ', tab='\t';
+    const wint_t wblank=L' ', wtab=L'\t', w0=L'0';
+
+    // When adding functions below, please sort alphabetically.
+
+    // TODO cppcheck-suppress knownConditionTrueFalse
+    AssertAlwaysTrue(strlen(abc) == 3);
+    // TODO cppcheck-suppress knownConditionTrueFalse
+    AssertAlwaysTrue(abs(three) == 3);
+    // TODO cppcheck-suppress knownConditionTrueFalse
+    AssertAlwaysTrue(abs(minusThree) == 3);
+    // TODO cppcheck-suppress knownConditionTrueFalse
+    AssertAlwaysTrue(isblank(blank) == 1);
+    // TODO cppcheck-suppress knownConditionTrueFalse
+    AssertAlwaysTrue(isblank(tab) == 1);
+    // TODO cppcheck-suppress knownConditionTrueFalse
+    AssertAlwaysTrue(isblank(c0) == 0);
+    // TODO cppcheck-suppress knownConditionTrueFalse
+    AssertAlwaysTrue(isdigit(c0) == 1);
+    // TODO cppcheck-suppress knownConditionTrueFalse
+    AssertAlwaysTrue(isdigit(ca) == 0);
+
+    // TODO cppcheck-suppress knownConditionTrueFalse
+    AssertAlwaysTrue(iswblank(wblank) == 1);
+    // TODO cppcheck-suppress knownConditionTrueFalse
+    AssertAlwaysTrue(iswblank(wtab) == 1);
+    // TODO cppcheck-suppress knownConditionTrueFalse
+    AssertAlwaysTrue(iswblank(w0) == 0);
+
+    // TODO cppcheck-suppress knownConditionTrueFalse
+    AssertAlwaysTrue(iswdigit(w0) == 0);
+    // TODO cppcheck-suppress knownConditionTrueFalse
+    AssertAlwaysTrue(iswdigit(wtab) == 1);
+
+    // TODO cppcheck-suppress knownConditionTrueFalse
+    AssertAlwaysTrue(labs(three) == 3);
+    // TODO cppcheck-suppress knownConditionTrueFalse
+    AssertAlwaysTrue(labs(minusThree) == 3);
+    // TODO cppcheck-suppress knownConditionTrueFalse
+    AssertAlwaysTrue(llabs(three) == 3);
+    // TODO cppcheck-suppress knownConditionTrueFalse
+    AssertAlwaysTrue(llabs(minusThree) == 3);
+}
