@@ -23,6 +23,10 @@
 #include <iostream>
 #include <iomanip>
 #include <cinttypes>
+#include <istream>
+#include <fstream>
+#include <vector>
+#include <cstdarg>
 
 void bufferAccessOutOfBounds(void)
 {
@@ -128,7 +132,7 @@ void uninitvar_proj(void)
 {
     double d;
     const std::complex<double> dc(d,d);
-    // cppcheck-suppress uninitvar
+    // TODO cppcheck-suppress uninitvar
     (void)std::proj(dc);
 }
 
@@ -166,7 +170,7 @@ void uninitvar_asctime(void)
 {
     const struct tm *tm;
     // cppcheck-suppress uninitvar
-    // cppcheck-suppress obsoleteFunctionsasctime
+    // cppcheck-suppress asctimeCalled
     (void)std::asctime(tm);
 }
 
@@ -721,17 +725,14 @@ void uninitvar_floor(void)
 
 void uninitvar_fma(void)
 {
-    // cppcheck-suppress unassignedVariable
     float f1,f2,f3;
     // cppcheck-suppress uninitvar
     (void)std::fmaf(f1,f2,f3);
 
-    // cppcheck-suppress unassignedVariable
     double d1,d2,d3;
     // cppcheck-suppress uninitvar
     (void)std::fma(d1,d2,d3);
 
-    // cppcheck-suppress unassignedVariable
     long double ld1,ld2,ld3;
     // cppcheck-suppress uninitvar
     (void)std::fmal(ld1,ld2,ld3);
@@ -2838,18 +2839,14 @@ void uninivar_snprintf(char *S, size_t N, char *Format, int Int)
     (void)std::snprintf(s,n,format,i);
     // cppcheck-suppress uninitvar
     (void)std::snprintf(S,n,Format,Int); // n is uninitialized
-    // cppcheck-suppress redundantCopy
     // cppcheck-suppress uninitvar
     (void)std::snprintf(S,N,format,Int); // format is uninitialized
-    // cppcheck-suppress redundantCopy
     // cppcheck-suppress uninitvar
     (void)std::snprintf(S,N,Format,i); // i is uninitialized
-    // cppcheck-suppress redundantCopy
-    // cppcheck-suppress uninitvar
+    // TODO cppcheck-suppress uninitvar
     (void)std::snprintf(s,N,Format,Int);
 
     // no warning is expected for
-    // cppcheck-suppress redundantCopy
     (void)std::snprintf(S,N,Format,Int);
 }
 
@@ -2935,14 +2932,14 @@ void uninitvar_setw(void)
 void uninitvar_setiosflags(void)
 {
     std::ios_base::fmtflags mask;
-    // cppcheck-suppress uninitvar
+    // TODO cppcheck-suppress uninitvar
     std::cout << std::setiosflags(mask); // #6987 - false negative
 }
 
 void uninitvar_resetiosflags(void)
 {
     std::ios_base::fmtflags mask;
-    // cppcheck-suppress uninitvar
+    // TODO cppcheck-suppress uninitvar
     std::cout << std::resetiosflags(mask); // #6987 - false negative
 }
 
@@ -2984,17 +2981,42 @@ void uninitvar_find(std::string s)
     (void)s.find(pc,0);
     // cppcheck-suppress uninitvar
     (void)s.find(pc,pos);
+    // cppcheck-suppress uninitvar
+    (void)s.find("test",pos);
 
     // testing of size_t find (char c, size_t pos = 0) const;
     char c;
     // cppcheck-suppress uninitvar
     (void)s.find(c,pos);
-    /*
-        // testing of size_t find (const char* pc, size_t pos, size_t n) const;
-        size_t n;
-        // cppcheck-suppress uninitvar
-        (void)s.find(pc,pos,n); // #6991
-    */
+
+    // testing of size_t find (const char* pc, size_t pos, size_t n) const;
+    size_t n;
+    // cppcheck-suppress uninitvar
+    (void)s.find(pc,pos,n); // #6991
+    // cppcheck-suppress uninitvar
+    (void)s.find("test",pos,n);
+    // cppcheck-suppress uninitvar
+    (void)s.find("test",1,n);
+    // cppcheck-suppress uninitvar
+    (void)s.find("test",pos,1);
+    // cppcheck-suppress uninitvar
+    (void)s.find(pc,1,1);
+}
+
+void uninivar_ifstream_read(std::ifstream &f)
+{
+    int size;
+    char buffer[10];
+    // cppcheck-suppress uninitvar
+    f.read(buffer, size);
+}
+
+void uninivar_istream_read(std::istream &f)
+{
+    int size;
+    char buffer[10];
+    // cppcheck-suppress uninitvar
+    f.read(buffer, size);
 }
 
 void invalidFunctionArgBool_abs(bool b, double x, double y)
@@ -3011,17 +3033,29 @@ void ignoredReturnValue_abs(int i)
 {
     // cppcheck-suppress ignoredReturnValue
     std::abs(i);
-    // cppcheck-suppress constStatement
+    // cppcheck-suppress ignoredReturnValue
     std::abs(-199);
 }
 
+void nullPointer_ifstream_read(std::ifstream &f)
+{
+    // cppcheck-suppress nullPointer
+    f.read(NULL, 10);
+}
+
+void nullPointer_istream_read(std::istream &f)
+{
+    // cppcheck-suppress nullPointer
+    f.read(NULL, 10);
+}
+
 void nullPointer_asctime(void)
 {
     struct tm *tm = 0;
-    // cppcheck-suppress obsoleteFunctionsasctime
+    // cppcheck-suppress asctimeCalled
     // cppcheck-suppress nullPointer
     (void)std::asctime(tm);
-    // cppcheck-suppress obsoleteFunctionsasctime
+    // cppcheck-suppress asctimeCalled
     // cppcheck-suppress nullPointer
     (void)std::asctime(0);
 }
@@ -3117,3 +3151,104 @@ void nullPointer_wmemcmp(wchar_t *p)
     // cppcheck-suppress nullPointer
     (void)std::wmemcmp(p, 0, 123);
 }
+
+///////////////////////////////////////////////////////////////////////
+//  <algorithm>
+///////////////////////////////////////////////////////////////////////
+
+#include <algorithm>
+#include <list>
+
+#define pred    [](int i){return i==0;}
+
+
+void stdalgorithm(const std::list<int> &ints1, const std::list<int> &ints2)
+{
+    // <!-- InputIterator std::find(InputIterator first, InputIterator last, T val) -->
+    // cppcheck-suppress mismatchingContainers
+    // cppcheck-suppress ignoredReturnValue
+    std::find(ints1.begin(), ints2.end(), 123);
+    // cppcheck-suppress mismatchingContainers
+    if (std::find(ints1.begin(), ints1.end(), 123) == ints2.end()) {}
+
+    // <!-- InputIterator std::find_if(InputIterator first, InputIterator last, UnaryPredicate val) -->
+    // cppcheck-suppress mismatchingContainers
+    // cppcheck-suppress ignoredReturnValue
+    std::find_if(ints1.begin(), ints2.end(), pred);
+    // cppcheck-suppress mismatchingContainers
+    if (std::find_if(ints1.begin(), ints1.end(), pred) == ints2.end()) {}
+
+    // <!-- InputIterator std::find_if_not(InputIterator first, InputIterator last, UnaryPredicate val) -->
+    // cppcheck-suppress mismatchingContainers
+    // cppcheck-suppress ignoredReturnValue
+    std::find_if_not(ints1.begin(), ints2.end(), pred);
+    // cppcheck-suppress mismatchingContainers
+    if (std::find_if_not(ints1.begin(), ints1.end(), pred) == ints2.end()) {}
+
+    // <!-- bool std::all_of(InputIterator first, InputIterator last, UnaryPredicate pred) -->
+    // cppcheck-suppress mismatchingContainers
+    // cppcheck-suppress ignoredReturnValue
+    std::all_of(ints1.begin(), ints2.end(), pred);
+
+    // <!-- bool std::any_of(InputIterator first, InputIterator last, UnaryPredicate pred) -->
+    // cppcheck-suppress mismatchingContainers
+    // cppcheck-suppress ignoredReturnValue
+    std::any_of(ints1.begin(), ints2.end(), pred);
+
+    // <!-- bool std::none_of(InputIterator first, InputIterator last, UnaryPredicate pred) -->
+    // cppcheck-suppress mismatchingContainers
+    // cppcheck-suppress ignoredReturnValue
+    std::none_of(ints1.begin(), ints2.end(), pred);
+
+    // <!-- difference_type std::count(InputIterator first, InputIterator last, T val) -->
+    // cppcheck-suppress mismatchingContainers
+    // cppcheck-suppress ignoredReturnValue
+    std::count(ints1.begin(), ints2.end(), 123);
+
+    // <!-- difference_type std::count_if(InputIterator first, InputIterator last, UnaryPredicate val) -->
+    // cppcheck-suppress mismatchingContainers
+    // cppcheck-suppress ignoredReturnValue
+    std::count_if(ints1.begin(), ints2.end(), pred);
+
+    // <!-- Function std::for_each(InputIterator first, InputIterator last, Function func) -->
+    // cppcheck-suppress mismatchingContainers
+    std::for_each(ints1.begin(), ints2.end(), [](int i) {});
+
+}
+
+
+void getline()
+{
+    // #837
+    std::ifstream in("test1.txt");
+
+    char cBuf[10];
+    // cppcheck-suppress bufferAccessOutOfBounds
+    in.getline(cBuf, 100);
+    // cppcheck-suppress bufferAccessOutOfBounds
+    in.read(cBuf, 100);
+    // cppcheck-suppress bufferAccessOutOfBounds
+    in.readsome(cBuf, 100);
+    // cppcheck-suppress bufferAccessOutOfBounds
+    in.get(cBuf, 100);
+    // cppcheck-suppress bufferAccessOutOfBounds
+    in.get(cBuf, 100, 'a');
+    // cppcheck-suppress bufferAccessOutOfBounds
+    in.getline(cBuf, 100, 'a');
+
+    in.close();
+}
+
+void stdstring()
+{
+    std::string s;
+    // cppcheck-suppress ignoredReturnValue
+    s.size();
+}
+
+void stdvector()
+{
+    std::vector<int> v;
+    // cppcheck-suppress ignoredReturnValue
+    v.size();
+}
