@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2015 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2018 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,9 +16,11 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include "tokenize.h"
+
 #include "checkautovariables.h"
+#include "settings.h"
 #include "testsuite.h"
+#include "tokenize.h"
 
 
 class TestAutoVariables : public TestFixture {
@@ -79,15 +81,21 @@ private:
         TEST_CASE(testautovar_return1);
         TEST_CASE(testautovar_return2);
         TEST_CASE(testautovar_return3);
-        TEST_CASE(testautovar_return4); // ticket #3030
         TEST_CASE(testautovar_extern);
         TEST_CASE(testinvaliddealloc);
         TEST_CASE(testinvaliddealloc_C);
         TEST_CASE(testassign1);  // Ticket #1819
         TEST_CASE(testassign2);  // Ticket #2765
 
+        TEST_CASE(assignAddressOfLocalArrayToGlobalPointer);
+        TEST_CASE(assignAddressOfLocalVariableToGlobalPointer);
+
         TEST_CASE(returnLocalVariable1);
         TEST_CASE(returnLocalVariable2);
+        TEST_CASE(returnLocalVariable3); // &x[0]
+        TEST_CASE(returnLocalVariable4); // x+y
+        TEST_CASE(returnLocalVariable5); // cast
+        TEST_CASE(returnLocalVariable6); // valueflow
 
         // return reference..
         TEST_CASE(returnReference1);
@@ -326,6 +334,16 @@ private:
               "    ptr++;\n"
               "}");
         ASSERT_EQUALS("[test.cpp:2]: (warning) Assignment of function parameter has no effect outside the function. Did you forget dereferencing it?\n", errout.str());
+
+        check("void foo(int* ptr) {\n" // #3177
+              "    --ptr;\n"
+              "}");
+        ASSERT_EQUALS("[test.cpp:2]: (warning) Assignment of function parameter has no effect outside the function. Did you forget dereferencing it?\n", errout.str());
+
+        check("void foo(struct S* const x) {\n" // #7839
+              "    ++x->n;\n"
+              "}");
+        ASSERT_EQUALS("", errout.str());
     }
 
     void testautovar11() { // #4641 - fp, assign local struct member address to function parameter
@@ -444,7 +462,7 @@ private:
 
     void testautovar_return2() {
         check("class Fred {\n"
-              "    int* func1()\n"
+              "    int* func1();\n"
               "}\n"
               "int* Fred::func1()\n"
               "{\n"
@@ -464,23 +482,6 @@ private:
         ASSERT_EQUALS("", errout.str());
     }
 
-    void testautovar_return4() {
-        // #3030
-        check("char *foo()\n"
-              "{\n"
-              "    char q[] = \"AAAAAAAAAAAA\";\n"
-              "    return &q[1];\n"
-              "}");
-        ASSERT_EQUALS("[test.cpp:4]: (error) Address of an auto-variable returned.\n", errout.str());
-
-        check("char *foo()\n"
-              "{\n"
-              "    static char q[] = \"AAAAAAAAAAAA\";\n"
-              "    return &q[1];\n"
-              "}");
-        ASSERT_EQUALS("", errout.str());
-    }
-
     void testautovar_extern() {
         check("struct foo *f()\n"
               "{\n"
@@ -626,6 +627,40 @@ private:
         ASSERT_EQUALS("", errout.str());
     }
 
+    void assignAddressOfLocalArrayToGlobalPointer() {
+        check("int *p;\n"
+              "void f() {\n"
+              "  int x[10];\n"
+              "  p = x;\n"
+              "}");
+        ASSERT_EQUALS("[test.cpp:4]: (warning) Address of local array x is assigned to global pointer p and not reassigned before x goes out of scope.\n", errout.str());
+
+        check("int *p;\n"
+              "void f() {\n"
+              "  int x[10];\n"
+              "  p = x;\n"
+              "  p = 0;\n"
+              "}");
+        ASSERT_EQUALS("", errout.str());
+    }
+
+    void assignAddressOfLocalVariableToGlobalPointer() {
+        check("int *p;\n"
+              "void f() {\n"
+              "  int x;\n"
+              "  p = &x;\n"
+              "}");
+        ASSERT_EQUALS("[test.cpp:4]: (warning) Address of local variable x is assigned to global pointer p and not reassigned before x goes out of scope.\n", errout.str());
+
+        check("int *p;\n"
+              "void f() {\n"
+              "  int x;\n"
+              "  p = &x;\n"
+              "  p = 0;\n"
+              "}");
+        ASSERT_EQUALS("", errout.str());
+    }
+
     void returnLocalVariable1() {
         check("char *foo()\n"
               "{\n"
@@ -634,6 +669,14 @@ private:
               "}");
         ASSERT_EQUALS("[test.cpp:4]: (error) Pointer to local array variable returned.\n", errout.str());
 
+        check("char *foo()\n" // use ValueFlow
+              "{\n"
+              "    char str[100] = {0};\n"
+              "    char *p = str;\n"
+              "    return p;\n"
+              "}");
+        ASSERT_EQUALS("[test.cpp:5]: (error) Pointer to local array variable returned.\n", errout.str());
+
         check("class Fred {\n"
               "    char *foo();\n"
               "};\n"
@@ -643,6 +686,16 @@ private:
               "    return str;\n"
               "}");
         ASSERT_EQUALS("[test.cpp:7]: (error) Pointer to local array variable returned.\n", errout.str());
+
+        check("char * format_reg(char *outbuffer_start) {\n"
+              "    return outbuffer_start;\n"
+              "}\n"
+              "void print_with_operands() {\n"
+              "    char temp[42];\n"
+              "    char *tp = temp;\n"
+              "    tp = format_reg(tp);\n"
+              "}");
+        ASSERT_EQUALS("", errout.str());
     }
 
     void returnLocalVariable2() {
@@ -664,6 +717,54 @@ private:
         ASSERT_EQUALS("", errout.str());
     }
 
+
+    void returnLocalVariable3() { // &x[..]
+        // #3030
+        check("char *foo() {\n"
+              "    char q[] = \"AAAAAAAAAAAA\";\n"
+              "    return &q[1];\n"
+              "}");
+        ASSERT_EQUALS("[test.cpp:3]: (error) Pointer to local array variable returned.\n", errout.str());
+
+        check("char *foo()\n"
+              "{\n"
+              "    static char q[] = \"AAAAAAAAAAAA\";\n"
+              "    return &q[1];\n"
+              "}");
+        ASSERT_EQUALS("", errout.str());
+    }
+
+    void returnLocalVariable4() { // x+y
+        check("char *foo() {\n"
+              "    char x[10] = {0};\n"
+              "    return x+5;\n"
+              "}");
+        ASSERT_EQUALS("[test.cpp:3]: (error) Pointer to local array variable returned.\n", errout.str());
+
+        check("char *foo(int y) {\n"
+              "    char x[10] = {0};\n"
+              "    return (x+8)-y;\n"
+              "}");
+        ASSERT_EQUALS("[test.cpp:3]: (error) Pointer to local array variable returned.\n", errout.str());
+    }
+
+    void returnLocalVariable5() { // cast
+        check("char *foo() {\n"
+              "    int x[10] = {0};\n"
+              "    return (char *)x;\n"
+              "}");
+        ASSERT_EQUALS("[test.cpp:3]: (error) Pointer to local array variable returned.\n", errout.str());
+    }
+
+    void returnLocalVariable6() { // valueflow
+        check("int *foo() {\n"
+              "    int x = 123;\n"
+              "    int p = &x;\n"
+              "    return p;\n"
+              "}");
+        ASSERT_EQUALS("[test.cpp:4]: (error) Address of auto-variable 'x' returned\n", errout.str());
+    }
+
     void returnReference1() {
         check("std::string &foo()\n"
               "{\n"
@@ -843,7 +944,6 @@ private:
               "    return foo();\n"
               "}");
         ASSERT_EQUALS("", errout.str());
-
         // Don't crash with function in unknown scope (#4076)
         check("X& a::Bar() {}"
               "X& foo() {"
@@ -970,6 +1070,11 @@ private:
               "    return \"foo\" + str;\n"
               "}");
         ASSERT_EQUALS("", errout.str());
+
+        check("int& incValue(int& value) {\n"
+              "    return ++value;\n"
+              "}");
+        ASSERT_EQUALS("", errout.str());
     }
 
     void returnReferenceLambda() {
@@ -981,6 +1086,22 @@ private:
               "    });\n"
               "}");
         ASSERT_EQUALS("", errout.str());
+
+        // #5844
+        check("map<string,string> const &getVariableTable() {\n"
+              "static map<string,string> const s_var = []{\n"
+              "    map<string,string> var;\n"
+              "    return var;\n"
+              "  }();\n"
+              "return s_var;\n"
+              "}");
+        ASSERT_EQUALS("", errout.str());
+
+        // #7583
+        check("Command& foo() {\n"
+              "  return f([]() -> int { return 1; });\n"
+              "}");
+        ASSERT_EQUALS("", errout.str());
     }
 
     void returnReferenceInnerScope() {
