@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2015 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2017 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -44,6 +44,7 @@ private:
         TEST_CASE(internalError);
         TEST_CASE(orInComplexPattern);
         TEST_CASE(extraWhitespace);
+        TEST_CASE(checkRedundantTokCheck);
     }
 
     void check(const char code[]) {
@@ -309,6 +310,12 @@ private:
               "    return tok->next()->next()->str();\n"
               "}");
         ASSERT_EQUALS("[test.cpp:2]: (style) Call to 'Token::next()' followed by 'Token::str()' can be simplified.\n", errout.str());
+
+        check("void f() {\n"
+              "    return tok->previous()->next()->str();\n"
+              "}");
+        ASSERT_EQUALS("[test.cpp:2]: (style) Call to 'Token::previous()' followed by 'Token::next()' can be simplified.\n", errout.str());
+
     }
 
     void internalError() {
@@ -402,6 +409,93 @@ private:
               "}");
         ASSERT_EQUALS("[test.cpp:3]: (warning) Found extra whitespace inside Token::findsimplematch() call: \"foobar \"\n", errout.str());
     }
+
+    void checkRedundantTokCheck() {
+        // findsimplematch
+        check("void f() {\n"
+              "    const Token *tok;\n"
+              "    if(tok && Token::findsimplematch(tok, \"foobar\")) {};\n"
+              "}");
+        ASSERT_EQUALS("[test.cpp:3]: (style) Unnecessary check of \"tok\", match-function already checks if it is null.\n", errout.str());
+
+        // findmatch
+        check("void f() {\n"
+              "    const Token *tok;\n"
+              "    if(tok && Token::findmatch(tok, \"%str% foobar\")) {};\n"
+              "}");
+        ASSERT_EQUALS("[test.cpp:3]: (style) Unnecessary check of \"tok\", match-function already checks if it is null.\n", errout.str());
+
+        // Match
+        check("void f() {\n"
+              "    const Token *tok;\n"
+              "    if(tok && Token::Match(tok, \"5str% foobar\")) {};\n"
+              "}");
+        ASSERT_EQUALS("[test.cpp:3]: (style) Unnecessary check of \"tok\", match-function already checks if it is null.\n", errout.str());
+
+        // simpleMatch
+        check("void f() {\n"
+              "    const Token *tok;\n"
+              "    if(tok && Token::simpleMatch(tok, \"foobar\")) {};\n"
+              "}");
+        ASSERT_EQUALS("[test.cpp:3]: (style) Unnecessary check of \"tok\", match-function already checks if it is null.\n", errout.str());
+
+        // Match
+        check("void f() {\n"
+              "    const Token *tok;\n"
+              "    if(tok->previous() && Token::Match(tok->previous(), \"5str% foobar\")) {};\n"
+              "}");
+        ASSERT_EQUALS("[test.cpp:3]: (style) Unnecessary check of \"tok->previous()\", match-function already checks if it is null.\n", errout.str());
+
+        // don't report:
+        // tok->previous() vs tok
+        check("void f() {\n"
+              "    const Token *tok;\n"
+              "    if(tok->previous() && Token::Match(tok, \"5str% foobar\")) {};\n"
+              "}");
+        ASSERT_EQUALS("", errout.str());
+
+        // tok vs tok->previous())
+        check("void f() {\n"
+              "    const Token *tok;\n"
+              "    if(tok && Token::Match(tok->previous(), \"5str% foobar\")) {};\n"
+              "}");
+        ASSERT_EQUALS("", errout.str());
+
+        // tok->previous() vs tok->previous()->previous())
+        check("void f() {\n"
+              "    const Token *tok;\n"
+              "    if(tok->previous() && Token::Match(tok->previous()->previous(), \"5str% foobar\")) {};\n"
+              "}");
+        ASSERT_EQUALS("", errout.str());
+
+        // if a && fn(a) triggers, make sure !a || !fn(a) triggers as well!
+        check("void f() {\n"
+              "    const Token *tok;\n"
+              "    if(!tok || !Token::simpleMatch(tok, \"foobar\")) {};\n"
+              "}");
+        ASSERT_EQUALS("[test.cpp:3]: (style) Unnecessary check of \"tok\", match-function already checks if it is null.\n", errout.str());
+
+        // if tok || !Token::simpleMatch...
+        check("void f() {\n"
+              "    const Token *tok;\n"
+              "    if(tok || !Token::simpleMatch(tok, \"foobar\")) {};\n"
+              "}");
+        ASSERT_EQUALS("", errout.str());
+
+        // if !tok || Token::simpleMatch...
+        check("void f() {\n"
+              "    const Token *tok;\n"
+              "    if(!tok || Token::simpleMatch(tok, \"foobar\")) {};\n"
+              "}");
+        ASSERT_EQUALS("", errout.str());
+
+        // something more complex
+        check("void f() {\n"
+              "    const Token *tok;\n"
+              "    if(!tok->previous()->previous() || !Token::simpleMatch(tok->previous()->previous(), \"foobar\")) {};\n"
+              "}");
+        ASSERT_EQUALS("[test.cpp:3]: (style) Unnecessary check of \"tok->previous()->previous()\", match-function already checks if it is null.\n", errout.str());
+    }
 };
 
 REGISTER_TEST(TestInternal)
