@@ -1,6 +1,6 @@
 /*
  * Cppcheck - A tool for static C/C++ code analysis
- * Copyright (C) 2007-2015 Daniel Marjamäki and Cppcheck team.
+ * Copyright (C) 2007-2018 Cppcheck team.
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,7 +26,7 @@
 #include <sstream>
 
 #include "../cli/filelister.h"
-#include "../cli/pathmatch.h"
+#include "../lib/pathmatch.h"
 
 static std::string builddir(std::string filename)
 {
@@ -37,7 +37,7 @@ static std::string builddir(std::string filename)
 
 static std::string objfile(std::string cppfile)
 {
-    cppfile.erase(cppfile.rfind("."));
+    cppfile.erase(cppfile.rfind('.'));
     return builddir(cppfile + ".o");
 }
 
@@ -50,15 +50,15 @@ static void getDeps(const std::string &filename, std::vector<std::string> &depfi
     std::ifstream f(filename.c_str());
     if (! f.is_open()) {
         if (filename.compare(0, 4, "cli/") == 0 || filename.compare(0, 5, "test/") == 0)
-            getDeps("lib" + filename.substr(filename.find("/")), depfiles);
+            getDeps("lib" + filename.substr(filename.find('/')), depfiles);
         return;
     }
     if (filename.find(".c") == std::string::npos)
         depfiles.push_back(filename);
 
     std::string path(filename);
-    if (path.find("/") != std::string::npos)
-        path.erase(1 + path.rfind("/"));
+    if (path.find('/') != std::string::npos)
+        path.erase(1 + path.rfind('/'));
 
     std::string line;
     while (std::getline(f, line)) {
@@ -67,7 +67,7 @@ static void getDeps(const std::string &filename, std::vector<std::string> &depfi
             continue;
         pos1 += 10;
 
-        std::string::size_type pos2 = line.find("\"", pos1);
+        std::string::size_type pos2 = line.find('\"', pos1);
         std::string hfile(path + line.substr(pos1, pos2 - pos1));
         if (hfile.find("/../") != std::string::npos)    // TODO: Ugly fix
             hfile.erase(0, 4 + hfile.find("/../"));
@@ -78,13 +78,14 @@ static void getDeps(const std::string &filename, std::vector<std::string> &depfi
 static void compilefiles(std::ostream &fout, const std::vector<std::string> &files, const std::string &args)
 {
     for (unsigned int i = 0; i < files.size(); ++i) {
+        bool external(files[i].compare(0,10,"externals/") == 0);
         fout << objfile(files[i]) << ": " << files[i];
         std::vector<std::string> depfiles;
         depfiles.push_back("lib/cxx11emu.h");
         getDeps(files[i], depfiles);
         for (unsigned int dep = 0; dep < depfiles.size(); ++dep)
             fout << " " << depfiles[dep];
-        fout << "\n\t$(CXX) " << args << " $(CPPFLAGS) $(CFG) $(CXXFLAGS) $(UNDEF_STRICT_ANSI) -c -o " << objfile(files[i]) << " " << builddir(files[i]) << "\n\n";
+        fout << "\n\t$(CXX) " << args << " $(CPPFLAGS) $(CFG) $(CXXFLAGS)" << (external?" -w":"") << " $(UNDEF_STRICT_ANSI) -c -o " << objfile(files[i]) << " " << builddir(files[i]) << "\n\n";
     }
 }
 
@@ -112,45 +113,6 @@ static void makeConditionalVariable(std::ostream &os, const std::string &variabl
        << "\n";
 }
 
-static std::string getLibName(const std::string &path)
-{
-    // path can be e.g. "externals/foo/foo.cpp" then returned
-    // library name is "FOO".
-    std::string libName = path.substr(path.find('/')+1);
-    libName = libName.substr(0, libName.find('/'));
-    std::transform(libName.begin(), libName.end(),libName.begin(), ::toupper);
-    return libName;
-}
-
-static void makeExtObj(std::ostream &fout, const std::vector<std::string> &externalfiles)
-{
-    bool start = true;
-    std::ostringstream libNames;
-    std::string libName;
-    for (std::size_t i = 0; i < externalfiles.size(); ++i) {
-        if (start) {
-            libName = getLibName(externalfiles[i]);
-            fout << "ifndef " << libName << "\n";
-            fout << "    " << libName << " = " << objfile(externalfiles[i]);
-            libNames << "EXTOBJ += $(" << libName << ")\n";
-            start = false;
-        } else {
-            fout << std::string(14, ' ') << objfile(externalfiles[i]);
-        }
-
-        if (i+1 >= externalfiles.size() || libName != getLibName(externalfiles[i+1])) {
-            // This was the last file for this library
-            fout << "\nendif\n\n\n";
-            start = true;
-        } else {
-            // There are more files for this library
-            fout << " \\\n";
-        }
-    }
-
-    fout << libNames.str();
-}
-
 int main(int argc, char **argv)
 {
     const bool release(argc >= 2 && std::string(argv[1]) == "--release");
@@ -159,6 +121,10 @@ int main(int argc, char **argv)
     std::vector<std::string> libfiles;
     getCppFiles(libfiles, "lib/", false);
 
+    std::vector<std::string> extfiles;
+    extfiles.push_back("externals/simplecpp/simplecpp.cpp");
+    extfiles.push_back("externals/tinyxml/tinyxml2.cpp");
+
     std::vector<std::string> clifiles;
     getCppFiles(clifiles, "cli/", false);
 
@@ -173,33 +139,27 @@ int main(int argc, char **argv)
         return EXIT_FAILURE;
     }
 
-    std::vector<std::string> externalfiles;
-    getCppFiles(externalfiles, "externals/", true);
-
-
     // QMAKE - lib/lib.pri
     {
         std::ofstream fout1("lib/lib.pri");
         if (fout1.is_open()) {
             fout1 << "# no manual edits - this file is autogenerated by dmake\n\n";
             fout1 << "include($$PWD/pcrerules.pri)\n";
-            fout1 << "BASEPATH = ../externals/tinyxml/\n";
-            fout1 << "include($$PWD/../externals/tinyxml/tinyxml.pri)\n";
-            fout1 << "BASEPATH = ../lib/\n";
-            fout1 << "INCLUDEPATH += ../externals/tinyxml\n";
-            fout1 << "HEADERS += $${BASEPATH}check.h \\\n";
+            fout1 << "include($$PWD/../externals/externals.pri)\n";
+            fout1 << "INCLUDEPATH += $$PWD\n";
+            fout1 << "HEADERS += $${PWD}/check.h \\\n";
             for (unsigned int i = 0; i < libfiles.size(); ++i) {
                 std::string fname(libfiles[i].substr(4));
                 if (fname.find(".cpp") == std::string::npos)
                     continue;   // shouldn't happen
                 fname.erase(fname.find(".cpp"));
-                fout1 << std::string(11, ' ') << "$${BASEPATH}" << fname << ".h";
+                fout1 << std::string(11, ' ') << "$${PWD}/" << fname << ".h";
                 if (i + 1 < testfiles.size())
                     fout1 << " \\\n";
             }
             fout1 << "\n\nSOURCES += ";
             for (unsigned int i = 0; i < libfiles.size(); ++i) {
-                fout1 << "$${BASEPATH}" << libfiles[i].substr(4);
+                fout1 << "$${PWD}/" << libfiles[i].substr(4);
                 if (i < libfiles.size() - 1)
                     fout1 << " \\\n" << std::string(11, ' ');
             }
@@ -278,7 +238,7 @@ int main(int argc, char **argv)
          << "        CPPCHK_GLIBCXX_DEBUG=\n"
          << "    endif # !CPPCHK_GLIBCXX_DEBUG\n"
          << "\n"
-         << "    ifeq ($(MSYSTEM),MINGW32)\n"
+         << "    ifeq ($(MSYSTEM),MINGW32 MINGW64)\n"
          << "        LDFLAGS=-lshlwapi\n"
          << "    else\n"
          << "        RDYNAMIC=-lshlwapi\n"
@@ -315,7 +275,7 @@ int main(int argc, char **argv)
 
     // skip "-D_GLIBCXX_DEBUG" if clang, since it breaks the build
     makeConditionalVariable(fout, "CXX", "g++");
-    fout << "ifeq ($(CXX), clang++)\n"
+    fout << "ifeq (clang++, $(findstring clang++,$(CXX)))\n"
          << "    CPPCHK_GLIBCXX_DEBUG=\n"
          << "endif\n";
 
@@ -336,7 +296,6 @@ int main(int argc, char **argv)
                                 "-Wcast-qual "
 //                                "-Wconversion "  // danmar: gives fp. for instance: unsigned int sizeof_pointer = sizeof(void *);
                                 "-Wfloat-equal "
-                                "-Winline "
 //                                "-Wlogical-op "
                                 "-Wmissing-declarations "
                                 "-Wmissing-format-attribute "
@@ -346,18 +305,19 @@ int main(int argc, char **argv)
                                 "-Wredundant-decls "
                                 "-Wshadow "
 //                                "-Wsign-conversion "
-                                "-Wsign-promo "
+//                                "-Wsign-promo "
                                 "-Wno-missing-field-initializers "
                                 "-Wno-missing-braces "
 //                                "-Wunreachable-code "
                                 "-Wno-sign-compare "  // danmar: I don't like this warning, it's very rarelly a bug
+                                "-Wno-multichar "
                                 "$(CPPCHK_GLIBCXX_DEBUG) "
                                 "-g");
     }
 
-    fout << "ifeq ($(CXX), g++)\n"
+    fout << "ifeq (g++, $(findstring g++,$(CXX)))\n"
          << "    override CXXFLAGS += -std=c++0x\n"
-         << "else ifeq ($(CXX), clang++)\n"
+         << "else ifeq (clang++, $(findstring clang++,$(CXX)))\n"
          << "    override CXXFLAGS += -std=c++0x\n"
          << "else ifeq ($(CXX), c++)\n"
          << "    ifeq ($(shell uname -s), Darwin)\n"
@@ -376,9 +336,9 @@ int main(int argc, char **argv)
          << "endif\n\n";
 
     makeConditionalVariable(fout, "PREFIX", "/usr");
-    makeConditionalVariable(fout, "INCLUDE_FOR_LIB", "-Ilib -Iexternals/tinyxml");
-    makeConditionalVariable(fout, "INCLUDE_FOR_CLI", "-Ilib -Iexternals/tinyxml");
-    makeConditionalVariable(fout, "INCLUDE_FOR_TEST", "-Ilib -Icli -Iexternals/tinyxml");
+    makeConditionalVariable(fout, "INCLUDE_FOR_LIB", "-Ilib -Iexternals/simplecpp -Iexternals/tinyxml");
+    makeConditionalVariable(fout, "INCLUDE_FOR_CLI", "-Ilib -Iexternals/simplecpp -Iexternals/tinyxml");
+    makeConditionalVariable(fout, "INCLUDE_FOR_TEST", "-Ilib -Icli -Iexternals/simplecpp -Iexternals/tinyxml");
 
     fout << "BIN=$(DESTDIR)$(PREFIX)/bin\n\n";
     fout << "# For 'make man': sudo apt-get install xsltproc docbook-xsl docbook-xml on Linux\n";
@@ -391,6 +351,10 @@ int main(int argc, char **argv)
     for (size_t i = 1; i < libfiles.size(); ++i)
         fout << " \\\n" << std::string(14, ' ') << objfile(libfiles[i]);
     fout << "\n\n";
+    fout << "EXTOBJ =      " << objfile(extfiles[0]);
+    for (size_t i = 1; i < extfiles.size(); ++i)
+        fout << " \\\n" << std::string(14, ' ') << objfile(extfiles[i]);
+    fout << "\n\n";
     fout << "CLIOBJ =      " << objfile(clifiles[0]);
     for (size_t i = 1; i < clifiles.size(); ++i)
         fout << " \\\n" << std::string(14, ' ') << objfile(clifiles[i]);
@@ -400,49 +364,65 @@ int main(int argc, char **argv)
         fout << " \\\n" << std::string(14, ' ') << objfile(testfiles[i]);
     fout << "\n\n";
 
-    makeExtObj(fout, externalfiles);
-
-    fout << ".PHONY: run-dmake\n\n";
+    fout << ".PHONY: run-dmake tags\n\n";
     fout << "\n###### Targets\n\n";
     fout << "cppcheck: $(LIBOBJ) $(CLIOBJ) $(EXTOBJ)\n";
-    fout << "\t$(CXX) $(CPPFLAGS) $(CXXFLAGS) -o cppcheck $(CLIOBJ) $(LIBOBJ) $(EXTOBJ) $(LIBS) $(LDFLAGS) $(RDYNAMIC)\n\n";
+    fout << "\t$(CXX) $(CPPFLAGS) $(CXXFLAGS) -o $@ $^ $(LIBS) $(LDFLAGS) $(RDYNAMIC)\n\n";
     fout << "all:\tcppcheck testrunner\n\n";
-    fout << "testrunner: $(TESTOBJ) $(LIBOBJ) $(EXTOBJ) cli/threadexecutor.o cli/cmdlineparser.o cli/cppcheckexecutor.o cli/filelister.o cli/pathmatch.o\n";
-    fout << "\t$(CXX) $(CPPFLAGS) $(CXXFLAGS) -o testrunner $(TESTOBJ) $(LIBOBJ) cli/threadexecutor.o cli/cppcheckexecutor.o cli/cmdlineparser.o cli/filelister.o cli/pathmatch.o $(EXTOBJ) $(LIBS) $(LDFLAGS) $(RDYNAMIC)\n\n";
+    fout << "testrunner: $(TESTOBJ) $(LIBOBJ) $(EXTOBJ) cli/threadexecutor.o cli/cmdlineparser.o cli/cppcheckexecutor.o cli/filelister.o\n";
+    fout << "\t$(CXX) $(CPPFLAGS) $(CXXFLAGS) -o $@ $^ $(LIBS) $(LDFLAGS) $(RDYNAMIC)\n\n";
     fout << "test:\tall\n";
     fout << "\t./testrunner\n\n";
     fout << "check:\tall\n";
-    fout << "\t./testrunner -g -q\n\n";
-    fout << "checkcfg:\tcppcheck\n";
+    fout << "\t./testrunner -q\n\n";
+    fout << "checkcfg:\tcppcheck validateCFG validatePlatforms\n";
     fout << "\t./test/cfg/runtests.sh\n\n";
-    fout << "dmake:\ttools/dmake.o cli/filelister.o cli/pathmatch.o lib/path.o\n";
-    fout << "\t$(CXX) $(CXXFLAGS) -o dmake tools/dmake.o cli/filelister.o cli/pathmatch.o lib/path.o -Ilib $(LDFLAGS)\n\n";
+    fout << "dmake:\ttools/dmake.o cli/filelister.o $(SRCDIR)/pathmatch.o $(SRCDIR)/path.o externals/simplecpp/simplecpp.o\n";
+    fout << "\t$(CXX) $(CXXFLAGS) -o $@ $^ $(LDFLAGS)\n\n";
     fout << "run-dmake: dmake\n";
     fout << "\t./dmake\n\n";
-    fout << "reduce:\ttools/reduce.o externals/tinyxml/tinyxml2.o $(LIBOBJ)\n";
-    fout << "\t$(CXX) $(CPPFLAGS) $(CXXFLAGS) -g -o reduce tools/reduce.o -Ilib -Iexternals/tinyxml $(LIBOBJ) $(LIBS) externals/tinyxml/tinyxml2.o $(LDFLAGS) $(RDYNAMIC)\n\n";
+    fout << "generate_cfg_tests: tools/generate_cfg_tests.o $(EXTOBJ)\n";
+    fout << "\tg++ -Iexternals/tinyxml -o generate_cfg_tests tools/generate_cfg_tests.o $(EXTOBJ)\n";
+    fout << "reduce:\ttools/reduce.o $(LIBOBJ) $(EXTOBJ)\n";
+    fout << "\t$(CXX) $(CPPFLAGS) $(CXXFLAGS) -o $@ $^ $(LIBS) $(LDFLAGS) $(RDYNAMIC)\n\n";
     fout << "clean:\n";
-    fout << "\trm -f build/*.o lib/*.o cli/*.o test/*.o tools/*.o externals/tinyxml/*.o testrunner reduce dmake cppcheck cppcheck.1\n\n";
+    fout << "\trm -f build/*.o lib/*.o cli/*.o test/*.o tools/*.o externals/*/*.o testrunner reduce dmake cppcheck cppcheck.1\n\n";
     fout << "man:\tman/cppcheck.1\n\n";
     fout << "man/cppcheck.1:\t$(MAN_SOURCE)\n\n";
     fout << "\t$(XP) $(DB2MAN) $(MAN_SOURCE)\n\n";
     fout << "tags:\n";
-    fout << "\tctags -R --exclude=doxyoutput .\n\n";
+    fout << "\tctags -R --exclude=doxyoutput --exclude=test/cfg cli externals gui lib test\n\n";
     fout << "install: cppcheck\n";
     fout << "\tinstall -d ${BIN}\n";
     fout << "\tinstall cppcheck ${BIN}\n";
+    fout << "\tinstall addons/*.py ${BIN}\n";
+    fout << "\tinstall addons/*/*.py ${BIN}\n";
     fout << "\tinstall htmlreport/cppcheck-htmlreport ${BIN}\n";
     fout << "ifdef CFGDIR \n";
     fout << "\tinstall -d ${DESTDIR}${CFGDIR}\n";
     fout << "\tinstall -m 644 cfg/* ${DESTDIR}${CFGDIR}\n";
     fout << "endif\n\n";
+    fout << "# Validation of library files:\n";
+    fout << "ConfigFiles := $(wildcard cfg/*.cfg)\n";
+    fout << "ConfigFilesCHECKED := $(patsubst %.cfg,%.checked,$(ConfigFiles))\n";
+    fout << ".PHONY: validateCFG\n";
+    fout << "%.checked:%.cfg\n";
+    fout << "\txmllint --noout --relaxng cfg/cppcheck-cfg.rng $<\n";
+    fout << "validateCFG: ${ConfigFilesCHECKED}\n\n";
+    fout << "# Validation of platforms files:\n";
+    fout << "PlatformFiles := $(wildcard platforms/*.xml)\n";
+    fout << "PlatformFilesCHECKED := $(patsubst %.xml,%.checked,$(PlatformFiles))\n";
+    fout << ".PHONY: validatePlatforms\n";
+    fout << "%.checked:%.xml\n";
+    fout << "\txmllint --noout --relaxng platforms/cppcheck-platforms.rng $<\n";
+    fout << "validatePlatforms: ${PlatformFilesCHECKED}\n\n";
 
     fout << "\n###### Build\n\n";
 
     compilefiles(fout, libfiles, "${INCLUDE_FOR_LIB}");
     compilefiles(fout, clifiles, "${INCLUDE_FOR_CLI}");
     compilefiles(fout, testfiles, "${INCLUDE_FOR_TEST}");
-    compilefiles(fout, externalfiles, "${INCLUDE_FOR_LIB}");
+    compilefiles(fout, extfiles, "");
     compilefiles(fout, toolsfiles, "${INCLUDE_FOR_LIB}");
 
     return 0;
